
   'PROGRAMA PARA SIMULAR DE MANERA GRAFICA UNA RED DE GAS CON REGIMEN
   'PERMANENTE, EMPLEANDO EL METODO DE NEWTON-RAPHSON Y LAS SUBRUTINAS
   'DECOMP Y SOLVE
   '(Algoritmo de solucin basado en un proyecto presentado por el M. en C.
   'J. Toms Limn)
	 
	  'Versin 1.0 (Programa diseado por J. Antonio Ruiz G., 1992)

   'LISTA DE LAS PRINCIPALES VARIABLES EMPLEADAS EN EL SIMULADOR
    
	'SIMBOLO EN EL PROGRAMA                     DESCRIPCION
	
	   'A(I,J) ........................ Matriz de conectores.
	   'aaa, bbb ...................... Variables para cuantificar el tiempo,
								 'de mquina.
	   'CTE(I,J), CTE(J,I) ............ Matriz de coeficientes de transmi-
								 'sin.
	   'CX(I), CY(I) .................. Coordenadas X,Y del nodo I.
	   'D(I,J) ........................ Matriz de dimetros, [pg].
	   'DATO(I) ....................... Vector auxiliar.
	   'E ............................. Factor de eficiencia de la tubera.
	   'F(I) .......................... Vector de correcciones (vector solu-
								 'cin de la matriz de diseo).
	   'iter .......................... Nmero de iteracin.
	   'JA(I,J) ....................... Matriz de diseo ("Jacobiano").
	   'L(I,J) ........................ Matriz de longitudes, [km].
	   'ND ............................ Nmero de nodos.
	   'P(I) .......................... Vector de presiones, [lb/pg].
	   'Q(I) .......................... Vector de adicin o extraccin de,
								 'masa al sistema, [10^6 PCD].
	   'S(I,J), S(J,I) ................ Matriz de sentidos de flujo.
	   'TOL ........................... Tolerancia.
   
    DECLARE SUB archivo ()
    DECLARE SUB ayuda (selec)
    DECLARE SUB borrar ()
    DECLARE SUB calculos ()
    DECLARE SUB captura (letrero$, var!, min!, max!, ren%, col%, colcap%)
    DECLARE SUB conectores (CX(), CY(), A(), A$)
    DECLARE SUB decomp (NDIM, N, A(), COND, IPVT(), WORK())
    DECLARE SUB enrolla ()
    DECLARE SUB fijarnodos (X!, Y!)
    DECLARE SUB resultados ()
    DECLARE SUB solve (NDIM, N, A(), B(), IPVT())
   
    'Declarando variables de transferencia
	 COMMON SHARED ND, aaa, bbb, TOL, itera, Z, datarchivo
   
    'Dimensionando de manera esttica y comn al vector de coords.
	 DIM SHARED CX(50), CY(50)
   
    'Declarando constantes simblicas comunes a varias subrutinas
	 CONST enter = 13, escape = 27, downarrow = 80, uparrow = 72, delete = 83
	 CONST leftarrow = 75, rightarrow = 77, F1 = 59, F2 = 60, F3 = 61
   
    'Cursor invisible
	 LOCATE , , 0

    'Se invoca a la subrutina que genera una pantalla de presentacin
	 CALL enrolla
   
    'Al seleccionar SCREEN 1 se tienen (320 x 200) pixeles para grficos en
    'monitores VGA, CGA o EGA
	 SCREEN 1
   
    'Declarando el rea til del monitor y asignando color a la pantalla
10    CLS
	 CLEAR
	 VIEW PRINT 1 TO 25
	 COLOR 8, 1, 1
	 BEEP
	
    'Proteccin contra errores de underflow y overflow
	 ON ERROR GOTO 23
   
    'Factor de eficiencia de la tubera
	 E = .8
   
    'Tolerancia a considerar
	 TOL = .1
   
    'Despliegue del recuadro que delimita el rea de graficado
	 LOCATE 1, 2: PRINT "<F1>AYUDA  <F2>FIN SESION  <F3>EJEMPLO"
	 LINE (2, 10)-(318, 150), 1, B
   
    'Se genera el cursor en forma de cruceta y se fija su posicin inicial
	 X = 153
	 Y = 80
	 PSET (X, Y)
	 PSET (X + 2, Y)
	 PSET (X - 2, Y)
	 PSET (X, Y + 1)
	 PSET (X, Y - 1)
   
    'Se inicia el posicionamiento de los nodos
	 CALL fijarnodos(X, Y)
   
    'Se llama a la subrutina que limpia la parte inf. de la pantalla
	 CALL borrar
   
    'Dimensionando de manera dinmica y comn, las matrices y vectores
    'involucrados
	 DIM SHARED A(ND, ND), P(ND), Q(ND), D(ND, ND), L(ND, ND), S(ND, ND)
	 DIM SHARED WORK(ND), IPVT(ND), B(ND), CTE(ND, ND), JA(ND, ND), F(ND)
	 DIM SHARED DATO(ND)
   
    'Se trabaja con valores de archivo (15 nodos, ejemplo tomado de los
    'Apuntes de Computacin Aplicada a la Ingeniera Petrolera)
	 IF datarchivo = 1 THEN
	    'Se invoca la subrutina con valores de archivo
		 CALL archivo
	    GOTO 20
	 END IF
   
    LOCATE 1, 1: PRINT " < F1 > AYUDA      < F2 > FIN DE SESION "
   
    'Se invoca a la subrutina que conecta los nodos
	 CALL conectores(CX(), CY(), A(), "CON")
   
    'Se llama a la subrutina para borrado de letreros
	 CALL borrar
   
    'Modificacin de conectores
	 LOCATE 21, 2: PRINT "DESEA MODIFICAR ALGUN CONECTOR (s/n) ?";
	 L$ = INPUT$(1)
	 'Transformando a maysculas todos los caracteres
	   L$ = UCASE$(L$)
	
	 CALL borrar
	
	 IF L$ = "S" THEN CALL conectores(CX(), CY(), A(), "MOD")
	
	 CALL borrar
   
    'Captura de datos para conectores y nodos
	 LOCATE 1, 2: PRINT " CARACTERISTICAS DE NODOS Y CONECTORES"
	 FOR I = 1 TO ND
		LOCATE 21, 1: PRINT "nodo"; I
	    
		'Se invoca el subprograma de captura
		  CALL captura(":conoce PRESION o GASTO [1-2]", DATO(I), 1, 2, 21, 8, 31)
	    
		'Entrada de gastos y presiones supuestos y/o verdaderos
		IF DATO(I) = 1 THEN
		   'Se invoca el subprograma de captura
			CALL captura("P [lb/pg2] ", P(I), 0, 50000, 22, 8, 21)
			CALL captura("Q supuesto [MPCD] ", Q(I), -5000000, 5000000, 23, 8, 28)
		ELSE
		   'Se invoca el subprograma de captura
			CALL captura("Q [MPCD] ", Q(I), -5000000, 5000000, 22, 8, 19)
			CALL captura("P supuesta [lb/pg2] ", P(I), 0, 50000, 23, 8, 29)
		END IF
	    
		'Datos de los conectores (dimetros y longitudes)
		FOR J = I + 1 TO ND
		   
		    CALL borrar
		   
		    IF A(I, J) = 1 THEN
			  LOCATE 21, 5: PRINT "DIAM. Y LONG. DEL CONECTOR"; I; "-"; J
			  'Se invoca el subprograma de captura
			    CALL captura("DIAMETRO [pg] ", D(I, J), .001, 100, 22, 10, 26)
			    CALL captura("LONGITUD [km] ", L(I, J), .0001, 1000000, 23, 10, 26)
			  'Clculo del coef. de transmisin (ec. de Panhandle)
			    CTE(I, J) = .84269 * E * D(I, J) ^ 2.667 / (.6215 * L(I, J)) ^ .5
			    CTE(J, I) = CTE(I, J)
		    END IF
		   
		    CALL borrar
	    
		NEXT J
	
	 NEXT I
   
    CALL borrar
   
    'Se invoca a al subrutina que efecta los clculos
	 LOCATE 21, 4: PRINT "!! CALCULANDO, FAVOR DE ESPERAR !!"
	 LOCATE 23, 12: PRINT "ITERACION :"
	 CALL calculos

20  'Despliegue de un recuadro
	 LINE (0, 180)-(319, 195), 2, B
	 LOCATE 24, 2, 1: PRINT "< F2 > FIN     < ENTER > NUEVO CALCULO"
   
    'Entrada del mandato de trmino de sesin
    DO
	 K$ = INKEY$
    LOOP WHILE K$ = ""
    qy = ASC(RIGHT$(K$, 1))
   
    SELECT CASE qy
		 CASE F2
			 'Salida de sesin
			  CLS
			  END
		 CASE ELSE
			 'Nuevo clculo
			  GOTO 10
    END SELECT

   'Seccin de deteccin de errores
23   SOUND 720, 14
	CLS
	LOCATE 13, 2: PRINT "Error detectado: sistema incompatible"
	RESUME 25
25   END

SUB archivo
 
  'Subrutina con datos para 15 nodos, ejemplo tomado del Libro de Apuntes de
  'Computacin Aplicada a la Ingeniera Petrolera
  
  'Nm. de nodos, factor de efic. y tolerancia
    ND = 15
    E = .8
    TOL = .1
 
  'Matriz de conexiones
    A(1, 10) = 1: A(2, 11) = 1: A(3, 11) = 1: A(4, 12) = 1: A(5, 12) = 1
    A(6, 13) = 1: A(7, 8) = 1: A(7, 14) = 1: A(9, 10) = 1: A(10, 11) = 1
    A(11, 12) = 1: A(12, 13) = 1: A(13, 14) = 1: A(14, 15) = 1: A(10, 1) = 1
    A(11, 2) = 1: A(11, 3) = 1: A(12, 4) = 1: A(12, 5) = 1: A(13, 6) = 1
    A(8, 7) = 1: A(14, 7) = 1: A(10, 9) = 1: A(11, 10) = 1: A(12, 11) = 1
    A(13, 12) = 1: A(14, 13) = 1: A(15, 14) = 1
 
  'Vectores de coordenadas de los nodos
    CX(1) = 89: CY(1) = 64: CX(2) = 89: CY(2) = 80: CX(3) = 121: CY(3) = 96
    CX(4) = 153: CY(4) = 64: CX(5) = 153: CY(5) = 96: CX(6) = 185: CY(6) = 96
    CX(7) = 217: CY(7) = 96: CX(8) = 217: CY(8) = 112: CX(9) = 121: CY(9) = 48
    CX(10) = 121: CY(10) = 64: CX(11) = 121: CY(11) = 80: CX(12) = 153: CY(12) = 80
    CX(13) = 185: CY(13) = 80: CX(14) = 217: CY(14) = 80: CX(15) = 217: CY(15) = 64
 
  'Despliegue de la red
    FOR I = 1 TO ND
	  
	   'Se despliega un nodo en las coordenadas (X,Y) con radio 5
		CIRCLE (CX(I), CY(I)), 5
	  
	   'Transformacin de pixeles a columna y/o rengln
		col = CX(I) / 8
		col = INT(col)
		ren = CY(I) / 8
		ren = INT(ren)
	  
	   'Se despliega el nm. consecutivo para cada nodo
		LOCATE ren, col: PRINT I
	  
	   FOR J = 1 TO ND
		  IF A(I, J) = 1 THEN
		  'Se traza la lnea conectora
		    LINE (CX(I), CY(I))-(CX(J), CY(J)), 3
		  END IF
   
    NEXT J, I
 
  'Vectores de datos reales o supuestos, de presiones y de gastos. Si
  'dato(I)=1, el dato supuesto es el gasto, y viceversa si Dato(I)=2
    DATO(1) = 1: P(1) = 3200: Q(1) = -2900: DATO(2) = 1: P(2) = 4000: Q(2) = -3600
    DATO(3) = 1: P(3) = 4000: Q(3) = -3600: DATO(4) = 1: P(4) = 4000: Q(4) = -3100
    DATO(5) = 2: P(5) = 3990: Q(5) = -3000: DATO(6) = 2: P(6) = 3700: Q(6) = -1500
    DATO(7) = 1: P(7) = 3900: Q(7) = -1200: DATO(8) = 1: P(8) = 4200: Q(8) = -6000
    DATO(9) = 2: P(9) = 1000: Q(9) = 12000: DATO(10) = 2: P(10) = 3100: Q(10) = 0
    DATO(11) = 2: P(11) = 3880: Q(11) = 0: DATO(12) = 2: P(12) = 3900: Q(12) = 0
    DATO(13) = 2: P(13) = 3750: Q(13) = 0: DATO(14) = 2: P(14) = 3400: Q(14) = 0
    DATO(15) = 2: P(15) = 1000: Q(15) = 13000
 
  'Vectores de dimetros y de longitudes
    FOR I = 1 TO ND
	   FOR J = 1 + 1 TO ND
		  IF A(I, J) = 1 THEN
		    
			'Dimetros de 2 pg y longitudes de 2 km
			D(I, J) = 2
			L(I, J) = 2
		    
			'Dimetros de 4 pg
			IF I = 9 OR I = 10 OR I = 14 OR I = 15 THEN
			  D(10, 9) = 4: D(9, 10) = 4: D(14, 15) = 4: D(15, 14) = 4
			END IF
		    
			'Clculo del coef. de transmisin (ec. de Panhandle)
			 CTE(I, J) = .84269 * E * D(I, J) ^ 2.667 / (.6215 * L(I, J)) ^ .5
			 CTE(J, I) = CTE(I, J)
		
		 END IF
   
    NEXT J, I
 
  'Se llama a la subrutina que limpia la parte inf. de la pantalla
    CALL borrar
 
  'Se despliega un mensaje enmarcado
    LINE (45, 171)-(274, 187), 2, BF
    LOCATE 23, 7: PRINT "Presione cualquier tecla ..."
    DO
    LOOP WHILE INKEY$ = ""
 
  'Momentneamente se vara a formato tipo texto
    SCREEN 0
		  
  'Se designa el rea de desplegado
    VIEW PRINT 1 TO 25
    COLOR 15, 3, 3
    CLS
   
    'Se despliegan los datos de la red para gas
	 LOCATE 1, 1, 0: PRINT "Nota: signo negativo en gastos de prod."
	 LOCATE 3, 6: PRINT "Nodo   p[lb/pg]   Q[MPCD]"
	
	 FOR I = 1 TO ND
		LOCATE 3 + I, 7
		PRINT USING "##     ####.#     #####.#"; I; P(I); Q(I)
	 NEXT I
   
	 LOCATE 20, 3: PRINT "Dimetros ints. de 2 pg excepto en"
	 LOCATE 21, 3: PRINT "los conectores (9,10)  y  (14,15),"
	 LOCATE 22, 3: PRINT "que es de 4. Longitudes de  2  km."
   
    LOCATE 24, 7: PRINT "PRESIONE CUALQUIER TECLA ..."
    DO
    LOOP WHILE INKEY$ = ""
 
  'Se llama a la subrutina que limpia la parte inf. de la pantalla
    CALL borrar
 
  'Se invoca a al subrutina que efecta los clculos
    LOCATE 21, 4: PRINT "!! CALCULANDO, FAVOR DE ESPERAR !!"
    LOCATE 23, 12: PRINT "ITERACION :"
    CALL calculos

END SUB

SUB ayuda (selec)

 'Subrutina para el despliegue de ayuda
  
   'Despliegue del recuadro que delimita el rea de ayuda
	IF selec = 2 THEN
	  LINE (52, 156)-(268, 195), 2, BF
	ELSE
	  LINE (52, 163)-(268, 195), 2, BF
	END IF
  
   'Desplegado de letreros en funcin de la etapa en que se solicite la ayuda
	SELECT CASE selec
		  CASE 1
			  SOUND 700, 9
			  LOCATE 22, 8: PRINT "  !NO puede avanzar a la  "
			  LOCATE 23, 8: PRINT "  siguiente etapa, si no  "
			  LOCATE 24, 8: PRINT "   a elaborado una red !  "
		  CASE 2
			  LOCATE 21, 8: PRINT "<FLECHAS> desplazar cursor"
			  LOCATE 22, 8: PRINT " <DELETE> eliminar un nodo"
			  LOCATE 23, 8: PRINT " <ENTER>  ubicar un nodo  "
			  LOCATE 24, 8: PRINT "  <ESC>   siguiente etapa "
		  CASE 3
			  SOUND 700, 9
			  LOCATE 22, 8: PRINT "  !NO puede avanzar a la  "
			  LOCATE 23, 8: PRINT "  siguiente etapa, si no  "
			  LOCATE 24, 8: PRINT "   a conectado  la red !  "
		  CASE 4
			  LOCATE 22, 8: PRINT "  Digite los NUMEROS  de  "
			  LOCATE 23, 8: PRINT "   los nodos a conectar   "
			  LOCATE 24, 8: PRINT "  <ESC>  siguiente etapa  "
	END SELECT
    
    
   'Se muestra el letrero de ayuda hasta que se digita cualquier tecla
	DO
	LOOP WHILE INKEY$ = ""
  
   'Borrado del recuadro de ayuda
	IF selec = 2 THEN
	  LINE (52, 156)-(268, 195), 0, BF
	ELSE
	  LINE (52, 163)-(268, 195), 0, BF
	END IF

END SUB

SUB borrar

 'Subrutina que borra las lneas inferiores de despliegue de letreros
  
   'Se designa el rea de borrado
    VIEW PRINT
    FOR II = 1 TO 5
	   LOCATE 19 + II, 1
	   PRINT STRING$(40, 32)
    NEXT

END SUB

SUB calculos
 
  'Subrutina que obtiene la solucin del problema aplicando el Mtodo de
  'Newton-Raphson
  
   'Variable de tiempo (inicio del clculo iterativo)
	aaa = TIMER
  
   'Se inicializan variables y comienza la evaluacin
calc10: Z = 0#
  
   'Contador de iteraciones
	itera = itera + 1
  
   'Se obtiene el signo del sentido de flujo (matriz de sentidos de flujo)
	FOR I = 1 TO ND
	    FOR J = I + 1 TO ND
		   IF A(I, J) = 1 THEN
			 S(I, J) = (P(I) - P(J)) / ABS(P(I) - P(J)): S(J, I) = -S(I, J)
		   END IF
calc20:  NEXT J
	NEXT I
  
   'Se conforma el vector de trminos independientes
	FOR I = 1 TO ND
	    F(I) = Q(I)
	    FOR J = 1 TO ND
		   IF A(I, J) = 1 THEN
			 F(I) = F(I) + CTE(I, J) * S(I, J) * ABS(P(I) ^ 2 - P(J) ^ 2) ^ .5
		   END IF
	    NEXT J
	    F(I) = -F(I)
	NEXT I
  
   'Se constituye el JACOBIANO, es decir, la matriz de diseo JA(I,J)
	FOR I = 1 TO ND
	  
	   'Gastos dato (presiones supuestas)
	    IF DATO(I) = 2 THEN
		  FOR J = 1 TO ND
			 SUMA = 0#
			 IF I = J THEN
			    FOR K = 1 TO ND
				   IF A(I, K) = 1 THEN
					 SUMA = SUMA + CTE(I, K) / (ABS(P(I) ^ 2 - P(K) ^ 2)) ^ .5
				   END IF
			    NEXT K
			    JA(I, J) = SUMA * P(I)
			 ELSE
			    IF A(I, J) = 1 THEN
				  IF DATO(J) = 2 THEN
					JA(I, J) = -CTE(I, J) * P(J) / (ABS(P(I) ^ 2 - P(J) ^ 2)) ^ .5
				  ELSE
					JA(I, J) = 0#
				  END IF
			    END IF
			 END IF
		  NEXT J
	   
	    ELSE
	    
		'Presiones dato (gastos supuestos)
		  FOR J = 1 TO ND
			 IF I = J THEN
			    JA(I, J) = 1
			 ELSE
			    IF A(I, J) = 1 THEN
				  IF DATO(J) = 2 THEN
					JA(I, J) = -CTE(I, J) * S(I, J) * P(J) / (ABS(P(I) ^ 2 - P(J) ^ 2)) ^ .5
				  ELSE
					JA(I, J) = 0#
				  END IF
			    END IF
			 END IF
		  NEXT J
	   
	    END IF
    
	NEXT I
  
   'Se invoca a DECOMP
	CALL decomp(ND, ND, JA(), COND, IPVT(), WORK())
	
	'S la matriz es singular se aborta el proceso
	 IF COND = 1E+32 THEN
	   BEEP: CLS
	   COLOR 31
		LOCATE 13, 17: PRINT " MATRIZ SINGULAR SIN SOLUCION !!"
	   COLOR 7
	   END
	 END IF
    
	'Se llama a SOLVE para obtener la solucin del sistema (vector F)
	 CALL solve(ND, ND, JA(), F(), IPVT())
  
   'Se corrige el vector de trminos independientes (mtodo Newton-Raphson)
	FOR I = 1 TO ND
	   
	    'Gastos dato (presiones supuestas)
		 IF DATO(I) = 2 THEN
		    'Nota : considerando una modificacin a la Tcnica de Stoner
		    'para evitar la divergencia
			 IF itera <= 3 THEN
			    P(I) = P(I) + (F(I) / 2)
			 ELSE
			    P(I) = P(I) + F(I)
			 END IF
		
		 ELSE
		   
		    'Presiones dato (gastos supuestos)
		    'Nota : considerando una modificacin a la Tcnica de Stoner
		    'para evitar la divergencia
			 IF itera <= 3 THEN
			    Q(I) = Q(I) + (F(I) / 2)
			 ELSE
			    Q(I) = Q(I) + F(I)
			 END IF
		
		 END IF
		
		 'Se selecciona el mximo valor absoluto F(i)
		   IF ABS(F(I)) > Z THEN Z = ABS(F(I))
    
	NEXT I
  
   'Se imprime el nmero de iteracin
	LOCATE 23, 23: PRINT itera
  
   'Se verifica la tolerancia contra el mximo valor absoluto de F(i)
	IF Z > TOL THEN GOTO calc10
  
   'Variable de tiempo (final del clculo iterativo)
	bbb = TIMER
  
   BEEP
  
   'Se invoca a la subrutina que despliega los resultados
	CALL resultados

END SUB

SUB captura (letrero$, var!, min!, max!, ren%, col%, colcap%)
 
  'Subprograma que controla la captura de informacin
    'letrero$ -- frase a desplegar
    'var      -- nombre de la variable de entrada
    'min, max -- valores mnimo y mximo
    'ren, col -- rengln y columna de desplegado
    'colcap   -- columna de captura y de borrado en caso de error
  
    DO
	 LOCATE ren%, col%
	 PRINT letrero$; : INPUT var
	 'Se verifica que el dato introducido est dentro de un rango lgico
	 IF var < min OR var > max THEN
	    SOUND 300, 3: LOCATE ren%, colcap%: PRINT "          "
	 END IF
    LOOP WHILE var < min OR var > max

END SUB

SUB conectores (CX(), CY(), A(), A$)
 
  'Subrutina que traza o elimina los conectores entre nodos
  
   IF A$ = "MOD" THEN
	 'Letrero de borrado de conectores
	   B$ = "modificar"
	   C$ = "MODIFICAR"
	   E$ = "CONEXIONES"
	  'Color de borrado de conectores
	   C = 0
   ELSE
	 'Letrero de conexin de conectores
	   B$ = "conectar "
	   C$ = "CONEXION"
	   E$ = "DE LOS NODOS"
	 'Color de conexin de conectores
	   C = 3
   END IF

 'En funcin de la tecla oprimida se ejecuta una accin
   WHILE qy <> escape
	   
	    'Se imprime letrero
conec10: LOCATE 20, 10: PRINT C$; " "; E$
	    LOCATE 22, 12: PRINT "nodo a " + B$; "?";
	   
	    'Se proporciona informacin
	    DO
		 K$ = INKEY$
	    LOOP WHILE K$ = ""
	    'Transmutacin a cdigo ASCII
		qy = ASC(RIGHT$(K$, 1))
	   
	    SELECT CASE qy
		   CASE escape
			   IF IX = 0 OR JX = 0 THEN
				 CALL ayuda(3)
				 GOTO conec10
			   END IF
			   EXIT SUB
		  
		   CASE F1
			   CALL ayuda(4)
		  
		   CASE F2
			   CLS
			   END
		  
		   'Se aceptan valores numricos
		   CASE 45 TO 67
			   'Nodo donde inicia el conector y despliegue de letrero
				valor$ = CHR$(qy)
				LOCATE 22, 12: PRINT "nodo a " + B$; "? "; valor$
			   
			   'Se proporciona informacin
conec20:            DO
				  ky$ = INKEY$
				LOOP WHILE ky$ = ""
				kz = ASC(RIGHT$(ky$, 1))
				
				SELECT CASE kz
					  CASE enter
						  'Se asigna el valor real tecleado, a una
						  'variable
						    IX = VAL(valor$)
						    valor$ = ""
							
						  'Se verifica la autenticidad del nodo
						  'analizado
						    IF IX <= 0 OR IX > ND THEN
							  SOUND 200, 5
							  CALL borrar
							  GOTO conec10
						    END IF
							  
						  'Nodo donde finaliza el conector
conec30:                          LOCATE 23, 12: PRINT "con cual ? ";
						 
						  'Se proporciona informacin
						    DO
							 ky2$ = INKEY$
						    LOOP WHILE ky2$ = ""
						    Kz2 = ASC(RIGHT$(ky2$, 1))
							  
						    SELECT CASE Kz2
								 CASE enter
									 JX = VAL(valor2$)
									 valor2$ = ""
									
									 'Se checa autenticidad
									 'del nodo analizado
									   IF JX <= 0 OR JX > ND OR JX = IX THEN
										 SOUND 200, 5
										 CALL borrar
										 GOTO conec10
									   END IF
									
									 'Se imprime el nmero
									   LOCATE 23, 22: PRINT JX
									
									 'Se traza o se elimina la lnea
									 'conectora
									   LINE (CX(IX), CY(IX))-(CX(JX), CY(JX)), C
									
									 'Se conforma la matriz de
									 'conectores [si el conector
									 'existe A(IX,JX)=1]
									   IF A$ = "MOD" THEN
										 A(IX, JX) = 0
									   ELSE
										 A(IX, JX) = 1
									   END IF
									
									 'Se igualan elementos
									   A(JX, IX) = A(IX, JX)
									  
									 CALL borrar
								
								 'Se aceptan valores numricos
								 CASE 45 TO 67
									 valor2$ = valor2$ + CHR$(Kz2)
									 LOCATE 23, 23: PRINT valor2$
									 GOTO conec30
								
								 CASE ELSE
									 SOUND 200, 2
									 GOTO conec30
							  
						    END SELECT
					 
					  'Se aceptan valores numricos               
					  CASE 45 TO 67
						  valor$ = valor$ + CHR$(kz)
						  LOCATE 22, 30: PRINT valor$
						  GOTO conec20
				
				 END SELECT
		  
		   CASE ELSE
			   SOUND 200, 2
			   GOTO conec10
	   
	    END SELECT
  
   WEND

END SUB

SUB decomp (NDIM, N, A(), COND, IPVT(), WORK())
   
   'Descomposicin de una matriz real por Eliminacin Gaussiana, adems, se
   'estima su nmero de condicin.
   'Con la subrutina SOLVE se obtiene la solucin del sistema lineal.
  
   'Datos de entrada:
	 'NDIM = Dimensionamiento del arreglo de la matriz A.
	    'N = Orden de la matriz.
	    'A = Matriz que ser triangularizada.
  
   'Datos de salida:
	    'A = Contiene una matriz tringular superior  (U)  y  una  versin
		   'permutada de una tringular inferior (I-L), por lo tanto:
				   '(matriz permutada)*A = L*U.
	 'COND = Valor estimado del nmero de condicin de A. Para un  sistema
		   'lineal A*X = B, cambios en los valores  de  A  y/o  B, pueden
		   'causar modificaciones a COND y a veces en X. Si COND+1 = COND,
		   'A es una matriz singular an   al  trabajarla  con  precisin,
		   'entonces COND = 1.0D+32 si la singularidad exacta es detectada.
	 'IPVT = Vector pivote.
		   'IPVT(K) = El ndice del k-simo rengln pivote.
		   'IPVT(N) = (-1)^(nmero de intercambios).
  
    'Espacio de trabajo:
		  'El vector WORK debe declararse e incluirse en  la  llamada  a
		  'esta subrutina. Su contenido de entrada se ignora (cero) y el
		  'de salida no es relevante.
  
    'El determinante de A puede obtenerse mediante la expresin:
		'Det(A) = IPVT(N) * A(1,1) * A(2,2) * ... * A(N,N)
   
    IPVT(N) = 1
    IF N = 1 THEN GOTO d80
    NM1 = N - 1

    'Clculo de (1-NORMA) de A
	 ANORM = 0#
	 FOR J = 1 TO N
		T = 0#
		FOR I = 1 TO N
		    T = T + ABS(A(I, J))
		NEXT I
		IF T > ANORM THEN ANORM = T
	 NEXT J

    'Eliminacin Gaussiana con pivoteo parcial
	 FOR K = 1 TO NM1
		KP1 = K + 1
  
		'Se ubica el pivote
		M = K
		FOR I = KP1 TO N
		    IF ABS(A(I, K)) > ABS(A(M, K)) THEN M = I
		NEXT I
		IPVT(K) = M
		IF M <> K THEN IPVT(N) = -IPVT(N)
		T = A(M, K)
		A(M, K) = A(K, K)
		A(K, K) = T
   
		'Si el pivote es cero se efecta un salto
		IF T = 0# THEN GOTO d35
  
		'Clculo de multiplicaciones
		FOR I = KP1 TO N
		    A(I, K) = -A(I, K) / T
		NEXT I
  
		'Intercambio y eliminacin por columnas
		FOR J = KP1 TO N
		    T = A(M, J)
		    A(M, J) = A(K, J)
		    A(K, J) = T
		    IF T = 0# THEN GOTO d30
		    FOR I = KP1 TO N
			   A(I, J) = A(I, J) + A(I, K) * T
		    NEXT I
d30:      NEXT J
d35:  NEXT K
  
    'COND = (1-NORMA de A)*(estimacin de 1-NORMA de la inversa de A)
	 'La estimacin se obtiene en un paso iterativo inverso para el vector
	 'singular reducido. Esto involucra  la  solucin  de  2  sistemas  de
	 'ecuaciones, (TRANSPUESTA de A)*Y = E y A*Z = Y, donde E es un vector
	 'de eleccin (+1  -1) para adicionarse en Y.
	 'Estimacin = (1-NORMA de Z)/(1-NORMA de Y)
    
	 'Resolviendo (TRANSPUESTA de A)*Y = E
	 FOR K = 1 TO N
		T = 0#
		IF K = 1 THEN GOTO d45
		KM1 = K - 1
		FOR I = 1 TO KM1
		    T = T + A(I, K) * WORK(I)
		NEXT I
d45:      EK = 1#
		IF T < 0# THEN EK = -1#
		IF A(K, K) = 0# THEN GOTO d90
		WORK(K) = -(EK + T) / A(K, K)
	 NEXT K
	 FOR KB = 1 TO NM1
		K = N - KB
		T = 0#
		KP1 = K + 1
		FOR I = KP1 TO N
		    T = T + A(I, K) * WORK(K)
		NEXT I
		WORK(K) = T
		M = IPVT(K)
		IF M = K THEN GOTO d60
		T = WORK(M)
		WORK(M) = WORK(K)
		WORK(K) = T
d60:  NEXT KB
	 YNORM = 0
	 FOR I = 1 TO N
	   YNORM = YNORM + ABS(WORK(I))
	 NEXT I
  
    'Resolviendo A*Z = Y
	 CALL solve(NDIM, N, A(), WORK(), IPVT())
	 ZNORM = 0#
	 FOR I = 1 TO N
		ZNORM = ZNORM + ABS(WORK(I))
	 NEXT I
  
    'Estimando la condicin
	 COND = ANORM * ZNORM / YNORM
	 IF COND < 1# THEN COND = 1#
	 EXIT SUB
  
    'Sistema de 1 por 1
d80:  COND = 1#
	 IF A(1, 1) <> 0# THEN EXIT SUB
  
    'Singularidad exacta
d90:  COND = 1D+32
   
END SUB

SUB enrolla

  'Subrutina que despliega una cortina para presentacin de ttulos

   'Creacin de la cortina
	CLS
	VIEW PRINT
	COLOR 3, 1
	  FOR AR = 8 TO 70
		 FOR RA = 4 TO 21
			COLOR 7
			  LOCATE RA + 1, AR + 1: PRINT ""
			COLOR 3
			  LOCATE RA, AR: PRINT ""
	  NEXT RA, AR

   'Letreros
	COLOR 0, 3
	  LOCATE 7, 22: PRINT "T E C N I C A S    A V A N Z A D A S "
	  LOCATE 9, 25: PRINT "D E    P R O G R A M A C I O N"
	  LOCATE 13, 30: PRINT "C A P I T U L O   II"
	  LOCATE 16, 17: PRINT "Simulador grfico de redes de gas con rgimen"
	  LOCATE 17, 15: PRINT "permanente, emepleando el Mtodo de Newton-Raphson"

	COLOR 31, 0
	  LOCATE 24, 60: PRINT "ESPERE POR FAVOR ..."

   'Pausa de 2.5 segundos
	SLEEP 2.5

END SUB

SUB fijarnodos (X, Y)
 
  'Subrutina que controla el desplazamiento del cursor y que fija los nodos
 
   'En funcin de la tecla seleccionada se ejecuta una accin
   WHILE qy <> escape

fi1: 'Se despliegan, de manera estndar, las coordenadas del cursor
	  LOCATE 22, 1: PRINT "X ="; INT(X / 9)
	  LOCATE 23, 1: PRINT "Y ="; (9 - INT(Y / 16)) + 1
	'Se proporciona informacin
	 DO
	    K$ = INKEY$
	 LOOP WHILE K$ = ""
	 'Transmutacin a cdigo ASCII
	  qy = ASC(RIGHT$(K$, 1))
    
	SELECT CASE qy
	   CASE escape
		    'Siguiente etapa
		    IF ND = 0 THEN
			  CALL ayuda(1)
			  GOTO fi1
		    END IF
		    EXIT SUB
	  
	   CASE F1
		   'Ayuda
		   CALL ayuda(2)
	  
	   CASE F2
		   'Fin de sesin
		   CLS
		   END
	  
	   CASE F3
		   'Se trabaja con valores de archivo (15 nodos, ejemplo tomado de
		   'los Apuntes de Computacin Aplicada a la Ingeniera Petrolera)
			datarchivo = 1
			ND = 15
			EXIT SUB
	  
	   CASE rightarrow
		   'Desplazamiento hacia la derecha
		  
		   'Se delimita el desplazamiento del cursor
			IF X > 299 THEN
			   SOUND 100, 3
			   GOTO fi2
			END IF
		 
		   'Se borra la cruceta (cursor)
			GOSUB borracruceta
		 
		   'Incremento en el avance del cursor
			X = X + 16
		 
		   'Se traza la cruceta (cursor)
			GOSUB trazacruceta
		   
	   CASE leftarrow
		   'Desplazamiento hacia la izquierda
		    
		   'Se delimita el desplazamiento del cursor
			IF X < 25 THEN
			   SOUND 100, 3
			   GOTO fi2
			END IF
		  
		   'Se borra la cruceta (cursor)
			GOSUB borracruceta
		
		   'Decremento en el avance del cursor
			X = X - 16
		
		   'Se traza la cruceta (cursor)
			GOSUB trazacruceta
	    
	   CASE downarrow
		   'Desplazamiento hacia la abajo
		  
		   'Se delimita el desplazamiento del cursor
			IF Y > 129 THEN
			   SOUND 100, 3
			   GOTO fi2
			END IF
		 
		   'Se borra la cruceta (cursor)
			GOSUB borracruceta
	    
		   'Incremento en el avance del cursor
			Y = Y + 16
	    
		   'Se traza la cruceta (cursor)
			GOSUB trazacruceta
	    
	   CASE uparrow
		   'Desplazamiento hacia la arriba
		 
		   'Se delimita el desplazamiento del cursor
			IF Y < 25 THEN
			   SOUND 100, 3
			   GOTO fi2
			END IF
		
		   'Se borra la cruceta (cursor)
			GOSUB borracruceta
	   
		   'Decremento en el avance del cursor
			Y = Y - 16
	   
		   'Se traza la cruceta (cursor)
			GOSUB trazacruceta
	  
	   CASE enter
		   'Se evita marcar ms de una ocasin una misma posicin
		    IF X = Xaux AND Y = Yaux THEN
			  SOUND 500, 1
			  GOTO fi2
		    END IF
		  
		   'Se despliega el nodo (con radio 5), en las coordenadas (X,Y)
		    CIRCLE (X, Y), 5
		  
		   'Contador de nodos
		    ND = ND + 1
		    CX(ND) = X
		    CY(ND) = Y
		 
		   'Transformacin de pixeles a columna y/o rengln de las
		   'coordenadas para despliegue del nmero de nodo
		    col = X / 8: col = INT(col)
		    ren = Y / 8: ren = INT(ren)
		  
		   'Proteccin contra coordenadas invlidas del nmero de nodo
		    IF ren <= 0 THEN ren = 1
		    IF col <= 0 THEN col = 1
		 
		   'Se despliega el nmero consecutivo para cada nodo
		    LOCATE ren, col: PRINT ND
		 
		   'Variables auxiliares
		    Xaux = X
		    Yaux = Y
	  
	   CASE delete
		   'Proteccin cuando el nmero de nodo es nulo o se intenta borrar en
		   'ms de una ocasin el mismo nodo
		    IF ND = 0 OR (X = X2aux AND Y = Y2aux) THEN
			  SOUND 350, 1
			  GOTO fi2
		    END IF
		   
		   'Se elimina el nodo (con radio 5), en las coordsenadas (X,Y)
		    CIRCLE (X, Y), 5, 0
		  
		   'Se ajusta el contador de nodos
		    ND = ND - 1
		   
		   'Se elimina el nm. consecutivo para cada nodo
		    LINE (X - 3, Y - 10)-(X + 15, Y - 2), 0, BF
		  
		   'Variables auxiliares
		    X2aux = X
		    Y2aux = Y

fi2: END SELECT
  
   WEND

		   'Subrutina para borrar cruceta (cursor)
borracruceta:
			PRESET (X, Y)
			PRESET (X + 2, Y)
			PRESET (X - 2, Y)
			PRESET (X, Y - 1)
			PRESET (X, Y + 1)
RETURN
		  
		   'Subrutina para trazar cruceta (cursor)
trazacruceta:
			PSET (X, Y)
			PSET (X + 2, Y)
			PSET (X - 2, Y)
			PSET (X, Y - 1)
			PSET (X, Y + 1)
RETURN

END SUB

SUB resultados
 
  'Despliegue de resultados
  
    CLS
   
    'Se trabaj con valores de archivo, por ende, se retoman las condicones
    'de ambiente iniciales
	IF datarchivo = 1 THEN
	   SCREEN 1
	   COLOR 8, 1
	   VIEW PRINT 1 TO 25
	END IF
   
    'Letreros y despliegue de resultados
	LOCATE 2, 15: PRINT "RESULTADOS"
	LOCATE 4, 4: PRINT "NODO       PRESION         GASTO"
	LOCATE 5, 15: PRINT "[lb/pg2]        [MPCD]"
	FOR I = 1 TO ND
	    LOCATE 5 + I, 5
	    PRINT USING "##      ######.##      ######.##"; I; P(I); Q(I)
	NEXT
 
    'Despliegue de un recuadro en funcin del nmero de nodos
	ren = INT((5 + ND) * 8.5)
	LINE (15, 18)-(302, ren), 1, B
   
    'Se imprime el nmero de iteraciones
	LOCATE 22, 6: PRINT itera; "iteraciones en ";
	PRINT USING "###.##"; bbb - aaa; : PRINT " seg"

END SUB

SUB solve (NDIM, N, A(), B(), IPVT())
   
    'Solucin del sistema lineal, A*X = B.
    'S la subrutina DECOMP detect singularidad se omite la ejecucin de
    'este procedimiento.
  
    'Datos de entrada :
	   'NDIM = Dimensionamiento del arreglo de la matriz A.
		 'N = Orden de la matriz.
		 'A = Matriz tringularizada, obtenida por DECOMP.
		 'B = Vector de trminos independientes.
	   'IPVT = Vector pivote, obtenido por DECOMP.
  
    'Datos de salida :
		 'B = Vector solucin, X.

    'Eliminacin hacia adelante
	 IF N = 1 THEN GOTO s50
	 NM1 = N - 1
	 FOR K = 1 TO NM1
		KP1 = K + 1
		M = IPVT(K)
		T = B(M)
		B(M) = B(K)
		B(K) = T
		FOR I = KP1 TO N
		    B(I) = B(I) + A(I, K) * T
		NEXT I
	 NEXT K
  
    'Sustitucin hacia atrs
	 FOR KB = 1 TO NM1
		KM1 = N - KB
		K = KM1 + 1
		B(K) = B(K) / A(K, K)
		T = -B(K)
		FOR I = 1 TO KM1
		    B(I) = B(I) + A(I, K) * T
		NEXT I
	 NEXT KB

s50:  B(1) = B(1) / A(1, 1)

END SUB

