 
  'Simulador de flujo monofsico y transitorio a travs de medios porosos
  'en las direcciones X-Y, empleando el Esquema Implcito y el Algoritmo
  'de Descomposicin L-U.
            'Nota: Se consideran 6 celdas en ambas direcciones.
 
  'Versin 1.0 (programa originalmente presentado por alumnos de la
               'asignatura Tcnicas Avanzadas de Programacin, en el
               'semestre 91-II y modificado por J. Antonio Ruiz G., 1992)

    'LISTA DE LAS PRINCIPALES VARIABLES EMPLEADAS EN EL SIMULADOR
     
        'SIMBOLO EN EL PROGRAMA                  DEFINICION

         'coorX1, ..., coorX4 ..... Coordenada "x" de c/u de los 4 pozos
                                    'supuestos.
         'coorY1, ..., coorY4 ..... Coordenada "y" de c/u de los 4 pozos
                                    'supuestos.
         'Cte1, ..., Cte7 ......... Constantes de la ecuacin (2.12).
         'Ct ...................... Compresibilidad, [cm2/kg].
         'dx, dy .................. Dimensiones de las celdas en las
                                    'direcciones (x,y), [m].
         'dt ...................... Incremento de tiempo, [das].
         'h ....................... Espesor del medio poroso, [m].
         'i, j, T ................. Subndices para rengln, columna,
                                    'y tiempo, respectivamente.
         'kx, ky .................. Permeabilidad en las direcciones y
                                    'y x, [md].
         'mm1, mm2 ................ Dimensiones de los vectores de las
                                    'matrices L y U.
         'NcX, NcY ................ Nmero de celdas en las direcciones
                                    'y y x.
         'NN ...................... Dimensin de la matriz de coeficientes
         'Np ...................... Nmero de pozos, en este caso Np=4.
         'P() ..................... Matriz de presiones, [kg/cm].
         'Pb, Pi .................. Presiones de burbujeo e inicial,
                                    '[kg/cm abs].
         'poro .................... Porosidad, [fraccin].
         'Q() ..................... Matriz de gastos, [m3/da].
         'Rte1, ..., Rte4 ......... Gasto de c/u de los pozos, [m3/da].
         'Tsim .................... Tiempo de simulacin, [das].
         'U2(), ..., U7() ......... Diagonales de la matriz U.
         'vis ..................... Viscosidad, [cp].
         'L1(), ..., L7() ......... Diagonales de la matriz L.
   
    DECLARE SUB archidat ()
    DECLARE SUB calculos ()
    DECLARE SUB dimmax (X!)
    DECLARE SUB enrolla ()
    DECLARE SUB fin1 ()
    DECLARE SUB fin2 ()
    DECLARE SUB imparch1 ()
    DECLARE SUB imparch2 (X!, fld!, columna!)
    DECLARE SUB impredat (X!, X$, fld!, col1)
    DECLARE SUB impsolu (v!)
    DECLARE SUB lecdatos (col1!, row!, fld!, ky!, X!, X$)
    DECLARE SUB letreros ()
    DECLARE SUB malla ()
    DECLARE SUB manager ()
    DECLARE SUB matgen ()
    DECLARE SUB matini ()
    DECLARE SUB paginacion (v!)
    DECLARE SUB recuadro ()
    DECLARE FUNCTION rotar! (lower!, upper!, current!, inc!)
    DECLARE SUB solucion ()
    DECLARE SUB torre ()
   
    'Declarando las variables comunes a todas las subrutinas
    COMMON SHARED /variableyac1/ kx, ky, vis, Ctf, poro, dx, dy, dt, h, Pi, Pb
    COMMON SHARED /variableyac2/ NcX, NcY, Tsim, Np, NN, Rte(), numpant, Taux
    COMMON SHARED /variablearch1/ Rte1, Rte2, coorX1, coorY1, coorX2, coorY2
    COMMON SHARED /variablearch2/ Rte3, Rte4, coorX3, coorY3, coorX4, coorY4
    COMMON SHARED /variablecalc1/ P(), Q(), U2(), U3(), U4(), U5(), U6(), U7()
    COMMON SHARED /variablecalc2/ L1(), L2(), L3(), L4(), L5(), L6(), L7(), TI(), X(), y()
    COMMON SHARED /variablecalc3/ Cte1, Cte2, Cte3, Cte4, Cte5, Cte6, Cte7, impr
   
   'Cursor invisible
    LOCATE , , 0
    
   'Se invoca a la subrutina que genera una pantalla de presentacin
    CALL enrolla
   
    CLS
    CLEAR
    SOUND 235, 4
 
  'Proteccin contra errores de overflow y underflow
    ON ERROR GOTO f10

  'Dimensiones mximas de las direcs. Y y X
    NcX = 6
    NcY = 6
    NN = NcX * NcY

  'Dimensionando, de manera esttica y comn, al vector de gastos inicial
    DIM SHARED Rte(6, 6)

  'Por cuestiones prcticas, se fijar el nm. de pozos productores y/o
  'inyectores
    Np = 4

  'Dimensionando, de manera esttica y comn, (con un margen extra) a los
  'vectores de las matrices L y U, tambin, los de la sust. hacia adelante
  'y hacia atrs, Y y X respectivamente
    DIM SHARED U2(-10 TO NN + 10), U3(-10 TO NN + 10), U4(-10 TO NN + 10), U5(-10 TO NN + 10)
    DIM SHARED U6(-10 TO NN + 10), U7(-10 TO NN + 10), L1(-10 TO NN + 10), L2(-10 TO NN + 10)
    DIM SHARED L3(-10 TO NN + 10), L4(-10 TO NN + 10), L5(-10 TO NN + 10), L6(-10 TO NN + 10)
    DIM SHARED L7(-10 TO NN + 10), TI(-10 TO NN + 10), X(-10 TO NN + 10), y(-10 TO NN + 10)

  'Se invoca a las subrutinas que controlan la ejecucin del simulador y se
  'procede a dar solucin al problema
    CALL manager

  'Seccin de deteccin de errores
f10: SOUND 720, 14
     VIEW PRINT: CLS
     LOCATE 13, 10
     PRINT "Error detectado en los datos, favor de reiniciar la sesin !!"
     RESUME f20
f20: END

SUB archidat
 
  'Subrutina que lee los datos almacenados en un archivo secuencial
 
    'Se abre el archivo de datos
    'Nota: verificar que el archivo de datos se encuentre en el patrn de
    'Bsqueda indicado
      OPEN "A:\cap2\archivo.dat" FOR INPUT AS #1
 
    'Se leen los datos y dicho valor se asigna a la variable correspondiente
      INPUT #1, kx, ky, dx, dy, Pi, Pb, Ctf, poro, h, vis
      INPUT #1, dt, Tsim, Rte1, coorX1, coorY1
      INPUT #1, Rte2, coorX2, coorY2, Rte(3, 3), Rte(5, 5)
   
    'Se cierra al archivo
      CLOSE #1
   
    'Nm. de pginas a desplegar
      numpant = INT(Tsim / dt)
   
    'Considerando que la solucin se obtiene para un tiempo (t + 1)
      Taux = numpant + 1
   
    'Dimensionando de manera dinmica a la matriz de presiones P y a la de gastos Q
      DIM P(0 TO NcX, 0 TO NcY, 0 TO Taux), Q(0 TO NcX, 0 TO NcY, 0 TO Taux)

END SUB

SUB calculos
 
  'Subrutina que controla los clculos del simulador

    'Se invoca a la subrutina que crea la matriz de coefs. inicial
     CALL matini
   
    'Se invoca a la subrutina que genera las matrices L y U a distintos
    'tiempos
     CALL matgen
   
    'Se invoca a la subrutina que obtiene la solucin del sistema a
    'diferentes tiempos
     CALL solucion

END SUB

SUB dimmax (X)
 
  'Subrutina de dimensionamiento mximo de las coordenadas de los pozos
 
    'Si se supera el mx. valor para el dimensionamiento de la malla (6 x 6),
    'se rechaza el valor y se asigna el de 6
      SOUND 250, 7
      X = 6

END SUB

SUB enrolla

  'Subrutina que despliega una cortina para presentacin de ttulos

   'Creacin de la cortina
     CLS
     VIEW PRINT
     COLOR 3, 1
       FOR AR = 8 TO 70
           FOR RA = 4 TO 21
               COLOR 7
                 LOCATE RA + 1, AR + 1: PRINT ""
               COLOR 3
                 LOCATE RA, AR: PRINT ""
       NEXT RA, AR

   'Letreros
     COLOR 0, 3
       LOCATE 7, 22: PRINT "T E C N I C A S    A V A N Z A D A S "
       LOCATE 9, 25: PRINT "D E    P R O G R A M A C I O N"
       LOCATE 13, 30: PRINT "C A P I T U L O   II"
       LOCATE 16, 14: PRINT "Simulador de flujo monofsico y transitorio a travs"
       LOCATE 17, 20: PRINT "de medios porosos en las direcciones X-Y"
 
     COLOR 31, 0
       LOCATE 24, 60: PRINT "ESPERE POR FAVOR ..."

   'Pausa de 2.5 segundos
     SLEEP 2.5

END SUB

SUB fin1
 
  'Subrutina que controla el fin de sesin o la continuacin de los clculos
    
    'Se genera un espacio para desplegar informacin
     VIEW PRINT
     COLOR 31, 0
  
   'Eleccin de proseguir o finalizar
     LOCATE 23, 47: PRINT " <  Fin de sesin ?  (S) (N) > "
  
     'Se dgita una tecla y se define una accin
     DO
       sa$ = INKEY$
     LOOP WHILE sa$ = ""
    
     SELECT CASE sa$
            CASE "S", "s"
                 CALL fin2
            CASE ELSE
                 'Se restituye la pantalla
                 COLOR 14, 3
                 LOCATE 23, 47
                 FOR ii = 1 TO 32
                     PRINT CHR$(196);
                 NEXT
     END SELECT

END SUB

SUB fin2
 
  'Subrutina para finalizar la sesin
 
    COLOR 14, 3, 3
    CLS
    SOUND 300, 5
 
    'Se despliega una torre
     CALL torre
   
    'Se genera un recuadro de despedida
     COLOR 31
       LOCATE 22, 53: PRINT "FIN DE SESION"
       LOCATE 21, 46: PRINT ""
       LOCATE 21, 47: PRINT STRING$(26, "")
       LOCATE 21, 72: PRINT ""
       FOR i = 22 TO 23
           LOCATE i, 46: PRINT ""
           LOCATE i, 72: PRINT ""
       NEXT
       LOCATE 23, 46: PRINT ""
       LOCATE 23, 72: PRINT ""
       LOCATE 23, 47: PRINT STRING$(25, "")
     COLOR 3
 
    'Salida de sesin
     END

END SUB

SUB imparch1
 
  'Subrutina que lee los datos de archivo y los ordena para despliegue
       
    'Llamada a la subrutina que crea el recuadro principal
      CALL recuadro
   
    'Se invoca a la subrutina de despliege de letreros
      CALL letreros
   
    'Se llama a la subrutina que despliega, uno por uno, los valores de
    'archivo, considerando la columna (col1) y el rengln de despliegue (fld)
     
      'Datos del yac. y de la simulacin
      col1 = 32
        CALL imparch2(Ctf, 0, col1)
        CALL imparch2(kx, 2, col1)
        CALL imparch2(dx, 4, col1)
        CALL imparch2(Tsim, 6, col1)
        CALL imparch2(vis, 8, col1)
        CALL imparch2(Pi, 10, col1)
      col1 = 72
        CALL imparch2(h, 0, col1)
        CALL imparch2(ky, 2, col1)
        CALL imparch2(dy, 4, col1)
        CALL imparch2(dt, 6, col1)
        CALL imparch2(poro, 8, col1)
        CALL imparch2(Pb, 10, col1)
     
      'Datos de los pozos
      col1 = 29
        CALL imparch2(coorX1, 14, col1)
        CALL imparch2(coorX2, 15, col1)
        CALL imparch2(coorX3, 16, col1)
        CALL imparch2(coorX4, 17, col1)
      col1 = 43
        CALL imparch2(coorY1, 14, col1)
        CALL imparch2(coorY2, 15, col1)
        CALL imparch2(coorY3, 16, col1)
        CALL imparch2(coorY4, 17, col1)
      col1 = 63
        CALL imparch2(Rte1, 14, col1)
        CALL imparch2(Rte2, 15, col1)
        CALL imparch2(Rte3, 16, col1)
        CALL imparch2(Rte4, 17, col1)

END SUB

SUB imparch2 (X, fld, col1)
 
  'Subrutina que despliega en pantalla los valores de archivo
   
    'Rengln de inicio de despliegue
      row = 5
      IF X = 0 THEN
         'El formato vara para fld >= 14
           IF fld < 14 THEN
              X$ = "_______"
           ELSE
              SELECT CASE col1
                     CASE IS < 63
                          'Coordenadas de pozos
                          X$ = "___"
                     CASE ELSE
                          'Tpicos cualesquiera
                          X$ = "_______"
              END SELECT
           END IF
      ELSE
         'El formato vara para fld >= 1
           IF fld < 14 THEN
              'Cadena de 7 caracteres
              X$ = "_______" + STR$(X)
              X$ = RIGHT$(X$, 7)
           ELSE
              SELECT CASE col1
                     CASE 63
                          'Cadena de 7 caracteres
                          X$ = "_______" + STR$(X)
                          X$ = RIGHT$(X$, 7)
                     CASE ELSE
                          'Cadena de 3 caracteres
                          X$ = "___" + STR$(X)
                          X$ = RIGHT$(X$, 3)
              END SELECT
           END IF
      END IF
     
      'Se despliega la cadena de caracteres
      COLOR 0, 7
      LOCATE row + fld, col1: PRINT X$

END SUB

SUB impredat (X, X$, fld, col1)
 
  'Subrutina que despliega en pantalla el dato tecleado
   
    'Rengln en que inicia la impresin
      row = 5
      IF X = 0 THEN
         'Dato nulo
         SELECT CASE fld
                CASE 14 TO 17
                     IF col1 < 63 THEN
                       'Coordenadas nulas de pozos
                        X$ = "___"
                     ELSE
                       'Gastos nulos de pozos
                        X$ = "_______"
                     END IF
                CASE ELSE
                     'Tpicos nulos cualesquiera
                     X$ = "_______"
         END SELECT
      ELSE
         'El formato vara para fld >= 14
           IF fld < 14 THEN
              'Cadena de 7 caracteres
              X$ = "_______" + STR$(X)
              X$ = RIGHT$(X$, 7)
           ELSE
              SELECT CASE col1
                   CASE 63
                        'Cadena de 7 caracteres
                        X$ = "_______" + STR$(X)
                        X$ = RIGHT$(X$, 7)
                   CASE ELSE
                        'Cadena de 3 caracteres
                        X$ = "___" + STR$(X)
                        X$ = RIGHT$(X$, 3)
              END SELECT
           END IF
      END IF
     
      'Se despliega la cadena de caracteres
      COLOR 0, 7
      LOCATE row + fld, col1: PRINT X$;

END SUB

SUB impsolu (v)
 
  'Subrutina de despliegue de resultados, en funcin del nmero de pgina
  'seleccionada, es decir, en funcin del tiempo (Tiemp)
   
    COLOR 15, 3
    'Se inicializa variable auxiliar
      w = 0
   
    'Se ajusta la coordenada de tiempo (empleando el criterio de que para la
    'pantalla "n" se despliega la presin al tiempo "n-1"
      Tiemp = v - 1
   
    FOR i = 1 TO NcX
        'Se inicializa variable auxiliar
        w1 = 0
        FOR j = 1 TO NcY
            'Impresin de datos en forma de malla
            LOCATE 6 + w, 19 + 8 * w1
            PRINT USING "###.##"; P(i, j, Tiemp);
            w1 = w1 + 1
        NEXT j
        w = w + 2
    NEXT i
   
    COLOR 0, 3
      LOCATE 20, 16
      PRINT "Distribucin de la presin a un tiempo de "; (impr - 1) * dt; " das"

END SUB

SUB lecdatos (col1, row, fld, qy, X, X$)
 
  'Subrutina de lectura de datos, el valor tecleado se asigna a la variable
  'correspondiente
   
    COLOR 0, 7
    LOCATE row + fld, col1
   
    'Asignacin en funcin del rengln (fld) y columna (col1)
      SELECT CASE fld
             CASE 0
                IF col1 = 32 THEN
                   Ctf = X
                ELSE
                   h = X
                END IF
             CASE 2
                IF col1 = 32 THEN
                   kx = X
                ELSE
                   ky = X
                END IF
             CASE 4
                IF col1 = 32 THEN
                   dx = X
                ELSE
                   dy = X
                END IF
             CASE 6
                IF col1 = 32 THEN
                   Tsim = X
                ELSE
                   dt = X
                END IF
             CASE 8
                IF col1 = 32 THEN
                   vis = X
                ELSE
                   poro = X
                END IF
             CASE 10
                IF col1 = 32 THEN
                   Pi = X
                ELSE
                   Pb = X
                END IF
             CASE 14
                IF col1 = 29 THEN
                   'El mx. dimensionamiento no debe superarse
                     IF X > 6 THEN CALL dimmax(X)
                   coorX1 = X
                END IF
                IF col1 = 43 THEN
                   'El mx. dimensionamiento no debe superarse
                     IF X > 6 THEN CALL dimmax(X)
                   coorY1 = X
                END IF
                IF col1 = 63 THEN Rte1 = X
                Rte(coorX1, coorY1) = Rte1
             CASE 15
                IF col1 = 29 THEN
                   'El mx. dimensionamiento no debe superarse
                     IF X > 6 THEN CALL dimmax(X)
                   coorX2 = X
                END IF
                IF col1 = 43 THEN
                   'El mx. dimensionamiento no debe superarse
                     IF X > 6 THEN CALL dimmax(X)
                   coorY2 = X
                END IF
                IF col1 = 63 THEN Rte2 = X
                Rte(coorX2, coorY2) = Rte2
             CASE 16
                IF col1 = 29 THEN
                   'El mx. dimensionamiento no debe superarse
                     IF X > 6 THEN CALL dimmax(X)
                   coorX3 = X
                END IF
                IF col1 = 43 THEN
                   'El mx. dimensionamiento no debe superarse
                     IF X > 6 THEN CALL dimmax(X)
                   coorY3 = X
                END IF
                IF col1 = 63 THEN Rte3 = X
                Rte(coorX3, coorY3) = Rte3
             CASE 17
                IF col1 = 29 THEN
                   'El mx. dimensionamiento no debe superarse
                     IF X > 6 THEN CALL dimmax(X)
                   coorX4 = X
                END IF
                IF col1 = 43 THEN
                   'El mx. dimensionamiento no debe superarse
                     IF X > 6 THEN CALL dimmax(X)
                   coorY4 = X
                END IF
                IF col1 = 63 THEN Rte4 = X
                Rte(coorX4, coorY4) = Rte4
      END SELECT
   
    'Nuevo nm. de pginas a desplegar (se certifica que dt<>0)
      IF dt = 0 THEN dt = 1
      numpant = INT(Tsim / dt)
   
    'Reconsiderando que la solucin se obtiene para un tiempo (t + 1)
      Taux = numpant + 1
   
    'Redimensionando de manera dinmica a la matriz de presiones P y a la de
    'gastos Q
      REDIM P(0 TO NcX, 0 TO NcY, 0 TO Taux), Q(0 TO NcX, 0 TO NcY, 0 TO Taux)
   
    'Invocando a la subrutina que despliega datos en pantalla
      CALL impredat(X, X$, fld, col1)

END SUB

SUB letreros
 
  'Subrutina para desplegar letreros en la pantalla principal, se toma en
  'cuenta la posicin del letrero, su color y un formato para espacio sin
  'dato (lnea)
   
    'Color del desplegado
    COLOR 15, 3, 3
      LOCATE 3, 3: PRINT "Nm. de celdas direc. X      ";
    COLOR 0, 7
    'Color del rea de captura
    PRINT "____  6"
   
    'Color del desplegado
    COLOR 15, 3, 3
      LOCATE 3, 43: PRINT "Nm. de celdas direc. Y      ";
    'Color del rea de captura
    COLOR 0, 7
    PRINT "____  6"
   
    'Color del desplegado
    COLOR 15, 3, 3
      LOCATE 5, 3: PRINT "Compresibilidad    [cm/kg]  ";
    'Color del rea de captura
    COLOR 0, 7
    PRINT "_______"
   
    COLOR 15, 3, 3
      LOCATE 5, 43: PRINT "Espesor de la formacin  [m] ";
    COLOR 0, 7
    PRINT "_______"
   
    COLOR 15, 3, 3
      LOCATE 7, 3: PRINT "Permeabilidad direc. X  [md] ";
    COLOR 0, 7
    PRINT "_______"
   
    COLOR 15, 3, 3
      LOCATE 7, 43: PRINT "Permeabilidad direc. Y  [md] ";
    COLOR 0, 7
    PRINT "_______"
   
    COLOR 15, 3, 3
      LOCATE 9, 3: PRINT "Incremento direc. X      [m] ";
    COLOR 0, 7
    PRINT "_______"
   
    COLOR 15, 3, 3
      LOCATE 9, 43: PRINT "Incremento direc. Y      [m] ";
    COLOR 0, 7
    PRINT "_______"
   
    COLOR 15, 3, 3
      LOCATE 11, 3: PRINT "Tiempo de simulacin  [das] ";
    COLOR 0, 7
    PRINT "_______"
   
    COLOR 15, 3, 3
      LOCATE 11, 43: PRINT "Incremento de tiempo  [das] ";
    COLOR 0, 7
    PRINT "_______"
   
    COLOR 15, 3, 3
      LOCATE 13, 3: PRINT "Viscosidad              [cp] ";
    COLOR 0, 7
    PRINT "_______"
   
    COLOR 15, 3, 3
      LOCATE 13, 43: PRINT "Porosidad         [fraccin] ";
    COLOR 0, 7
    PRINT "_______"
   
    COLOR 15, 3, 3
      LOCATE 15, 3: PRINT "Presin inicial  [kg/cmabs] ";
    COLOR 0, 7
    PRINT "_______"
   
    COLOR 15, 3, 3
      LOCATE 15, 43: PRINT "P. de burbujeo   [kg/cmabs] ";
    COLOR 0, 7
    PRINT "_______"
   
    COLOR 15, 3, 3
      LOCATE 17, 5
      PRINT "El simulador considera 4 pozos (produc./inyec.). Gasto = 0 si no existe."
      LOCATE 18, 7
      PRINT "Por convencin, a un pozo productor se le asigna un gasto negativo."
      LOCATE 19, 12: PRINT "Pozo 1: coord. X ";
    COLOR 0, 7
    PRINT "___";
   
    COLOR 15, 3, 3
      LOCATE 19, 32: PRINT ", coord. Y ";
    COLOR 0, 7
    PRINT "___";
   
    COLOR 15, 3, 3
      LOCATE 19, 46: PRINT ", gasto [m3/da] ";
    COLOR 0, 7
    PRINT "_______";
   
    COLOR 15, 3, 3
      LOCATE 20, 12: PRINT "Pozo 2: coord. X ";
    COLOR 0, 7
    PRINT "___";
   
    COLOR 15, 3, 3
      LOCATE 20, 32: PRINT ", coord. Y ";
    COLOR 0, 7
    PRINT "___";
   
    COLOR 15, 3, 3
      LOCATE 20, 46: PRINT ", gasto [m3/da] ";
    COLOR 0, 7
    PRINT "_______";
   
    COLOR 15, 3, 3
      LOCATE 21, 12: PRINT "Pozo 3: coord. X ";
    COLOR 0, 7
    PRINT "___";
   
    COLOR 15, 3, 3
      LOCATE 21, 32: PRINT ", coord. Y ";
    COLOR 0, 7
    PRINT "___";
   
    COLOR 15, 3, 3
      LOCATE 21, 46: PRINT ", gasto [m3/da] ";
    COLOR 0, 7
    PRINT "_______";
   
    COLOR 15, 3, 3
      LOCATE 22, 12: PRINT "Pozo 4: coord. X ";
    COLOR 0, 7
    PRINT "___";
   
    COLOR 15, 3, 3
      LOCATE 22, 32: PRINT ", coord. Y ";
    COLOR 0, 7
    PRINT "___";
   
    'color del desplegado
    COLOR 15, 3, 3
      LOCATE 22, 46: PRINT ", gasto [m3/da] ";
    'color del rea de captura
    COLOR 0, 7
    PRINT "_______";

END SUB

SUB malla
 
  'Subrutina que genera una malla de 6 por 6 (cantidad fija de celdas),
  'adems, se despliegan las coordenadas de referencia en X y en Y
   
    'Se llama a la subrutina que crea el marco principal
      CALL recuadro
   
    'Se crea la malla
      COLOR 0, 3
        LOCATE 5, 17
        PRINT "Ŀ"
        FOR j = 0 TO 10 STEP 2
            LOCATE 6 + j, 17
            PRINT "                                          "
            LOCATE 7 + j, 17
            PRINT "Ĵ"
        NEXT
        LOCATE 17, 17
        PRINT ""
       
    'Se despliega el nm. de celdas en las direcs. Y y X
        FOR ii = 1 TO NcX
            LOCATE 4 + (2 * ii), 13: PRINT ii
            FOR jj = 1 TO NcY
                LOCATE 4, 12 + (8 * jj): PRINT jj
        NEXT jj, ii

END SUB

SUB manager
 
  'Subrutina que controla los desplegados en pantalla y la eleccin de la
  'alternativa de trabajo

    'Declarando constantes simblicas
    CONST enter = 13, escape = 27, pageup = 73, pagedown = 81, F1 = 59
    CONST downarrow = 80, uparrow = 72, leftarrow = 75, rightarrow = 77
   
    'Declarando el nm. de campos o renglones de captura (se incluyen los
    'intermedios, aunque estos no sean de captura)
    CONST fields = 17
   
    'Rengln en que se inicia la captura
    row = 5
   
    'Colunma en que se inicia la captura
    col1 = 32
   
    'Llamada a la subrutina que crea el recuadro principal
    CALL recuadro
   
    'Se invoca a la subrutina de despliege de letreros
    CALL letreros
   
    'Llamada a la subrutina de lectura de datos
    CALL archidat
   
    'Variable que permitir el despliegue del cursor en la pantalla de
    'captura
    impr = 1
   
    'Eleccin de una opcin de trabajo
    DO
     
      'Posicionamiento del cursor en cada campo (columna), en funcin del
      'rengln actual
        IF impr <> 1 THEN
           'Se invisibiliza al cursor en pantallas diferentes de la inicial
             LOCATE row + fld, col1, 0
        ELSE
          
           'Se permite el despliegue del cursor en la pantalla de captura
             IF fld <= 10 THEN
                'Desplazamiento por los renglones que cumplen con fld <= 10
                '(tpicos cualesquiera)
                SELECT CASE col1
                       CASE 32
                            IF qy = leftarrow THEN col1 = 72
                            IF qy = rightarrow THEN col1 = 72
                      
                       CASE ELSE
                            col1 = 72
                            IF qy = leftarrow THEN col1 = 32
                            IF qy = rightarrow THEN col1 = 32
                  END SELECT
             ELSE
                'Desplazamiento a travs de los renglones de los pozos
                '(fld >= 14)
                SELECT CASE col1
                       CASE 29
                            IF qy = leftarrow THEN col1 = 63
                            IF qy = rightarrow THEN col1 = 43
                      
                       CASE 43
                            IF qy = leftarrow THEN col1 = 29
                            IF qy = rightarrow THEN col1 = 63
                      
                       CASE ELSE
                            col1 = 63
                            IF qy = leftarrow THEN col1 = 43
                            IF qy = rightarrow THEN col1 = 29
                  END SELECT
             END IF
          
           'Despliegue, tamao y color del cursor en la pantalla de captura
           LOCATE row + fld, col1, 1, 5, 10
       
        END IF
     
      'Se captura informacin (el valor extendido de una tecla se convierte
      'a numrico para ejecutar una accin)
        DO
          K$ = INKEY$
        LOOP WHILE K$ = ""
        qy = ASC(RIGHT$(K$, 1))
       
        SELECT CASE qy
               CASE escape
                    'fin del programa
                      LOCATE , , 0
                      CALL fin1
              
               CASE uparrow, downarrow, leftarrow, rightarrow
                    'Se permite el despliegue del cursor en la pantalla
                    IF impr = 1 THEN
                       'Desplazamiento vertical del cursor de un campo de
                       'captura a otro (inc = salto de campo a campo)
                       IF fld < 10 THEN
                         
                          'Desplazamiento vertical (tpicos cualesquiera)
                          SELECT CASE qy
                                 CASE downarrow
                                      inc = 2
                                 CASE uparrow
                                      inc = -2
                                 CASE ELSE
                                      inc = 0
                          END SELECT
                      
                       ELSE
                          'Desplazamiento vertical (renglones de los pozos)
                          SELECT CASE fld
                                 CASE 10
                                      SELECT CASE qy
                                             CASE downarrow
                                                  inc = 4
                                             CASE uparrow
                                                  inc = -2
                                             CASE ELSE
                                                  inc = 0
                                          END SELECT
                                
                                 CASE 14
                                      SELECT CASE qy
                                             CASE downarrow
                                                  inc = 1
                                             CASE uparrow
                                                  inc = -4
                                             CASE ELSE
                                                  inc = 0
                                          END SELECT
                                
                                 CASE IS > 14
                                      SELECT CASE qy
                                                  CASE downarrow
                                                       inc = 1
                                                  CASE uparrow
                                                       inc = -1
                                                  CASE ELSE
                                                       inc = 0
                                      END SELECT
                         
                          END SELECT
                       END IF
                      
                       'Invocando la funcin que desplaza verticalmente
                       'al cursor
                         fld = rotar(0, fields, fld, inc)
                   
                    END IF
                
               'Captura e impresin de carcteres numricos exclusivamente
               CASE 45 TO 57
                    'Se permite el despliegue del cursor en la pantalla
                    IF impr = 1 THEN
                       entrada$ = CHR$(qy)
                       cont = 1
                      
                       'Se ajusta el color de la captura y se despliega
                         COLOR 0, 7
                           PRINT CHR$(qy)
                      
                       'Se posiciona la informacin en el espacio de captura
10 :                     LOCATE row + fld, col1
                        
                       'Se captura otro caracter
                         DO
                           ky$ = INKEY$
                         LOOP WHILE ky$ = ""
                        
                       'Se forma una cadena de caracteres (cifra) y si se
                       'presiona <enter> se almacena
                         kz = ASC(RIGHT$(ky$, 1))
                        
                         SELECT CASE kz
                                CASE enter
                                     X = VAL(entrada$)
                                    
                                     'Llamada a la subrutina que capta la
                                     'informacin tecleada
                                        LOCATE , , 0
                                        CALL lecdatos(col1, row, fld, qy, X, X$)
                               
                                CASE 45 TO 57
                                     'Se conforma la cadena de caracteres
                                     entrada$ = entrada$ + CHR$(kz)
                                    
                                     'Se posiciona la informacin en el
                                     'espacio de captura
                                     LOCATE row + fld, col1 + cont, 1
                                    
                                     'Se ajusta el color de la captura y se
                                     'despliega el caracter
                                       COLOR 0, 7
                                         PRINT CHR$(kz)
                                    
                                     cont = cont + 1
                                     GOTO 10
                               
                                CASE ELSE
                                     GOTO 10
                           END SELECT
                       
                        'Se posiciona el cursor
                        LOCATE row + fld, col1
                   
                    END IF
                
               CASE pagedown
                    'Se invisibiliza el cursor
                      LOCATE , , 0
                   
                    'Funcin que decrementa el cambio de pginas
                      impr = rotar(1, Taux, impr, -1)
                   
                    'Invocacin de la subrutina que cambia de pantalla
                      CALL paginacion(impr)
                
               CASE pageup
                    'Se invisibiliza el cursor
                      LOCATE , , 0
                   
                    'Funcin que incrementa el cambio de pginas
                      impr = rotar(1, Taux, impr, 1)
                   
                    'Se llama a la subrutina que cambia de pantalla
                      CALL paginacion(impr)
              
               CASE F1
                    'Se presenta un mensaje en pantalla
                      COLOR 31, 0
                        LOCATE 23, 5, 0: PRINT " < Efectuando clculos ... > ";
                   
                    'Llamado a la subrutina que realiza los clculos
                      CALL calculos
                      BEEP

                    'Se restituye la pantalla
                      COLOR 14, 3
                        LOCATE 23, 5
                        FOR ii = 1 TO 29
                            PRINT CHR$(196);
                        NEXT
                      COLOR 15
        END SELECT
        
        'Se invisibiliza el cursor
          LOCATE , , 0
   
    LOOP

END SUB

SUB matgen
 
  'Subrutina que genera la matriz de coeficientes  a distintos tiempos
    
     'Constantes de la ecuacin de flujo
     Cte1 = kx * h / (dx ^ 2)
     Cte2 = -2 * kx * h / (dx ^ 2)
     Cte3 = ky * h / (dy ^ 2)
     Cte4 = -2 * ky * h / (dy ^ 2)
     Cte5 = (-119.56 * poro * vis * Ctf * h) / dt
     Cte6 = -119.56 * vis / (dx * dy)
     Cte7 = Cte2 + Cte4 + Cte5
  
     'Calculando los elementos de las matrices del Algoritmo de
     'Descomposicin L-U
     FOR K = 1 TO NN
        
         IF (K = 1 OR K = NcY OR K = NN - (NcY - 1) OR K = NN) THEN
            'Esquinas de la matriz
             X1 = Cte7 + Cte1 + Cte3 - L7(K) * U7(K - 6) - L6(K) * U6(K - 5) - L5(K) * U5(K - 4)
             X2 = -L4(K) * U4(K - 3) - L3(K) * U3(K - 2) - L2(K) * U2(K - 1)
             L1(K) = X1 + X2
         ELSE
            
            IF K < NcY OR K > NN - (NcY - 1) THEN
               'Rengln sup. e inferior
                X1 = Cte7 + Cte1 - L7(K) * U7(K - 6) - L6(K) * U6(K - 5) - L5(K) * U5(K - 4)
                X2 = -L4(K) * U4(K - 3) - L3(K) * U3(K - 2) - L2(K) * U2(K - 1)
                L1(K) = X1 + X2
            ELSE
                
               IF (K = NcY + 1 OR K = NcY * 2 OR K = (NcY * 2) + 1 OR K = NcY * 3 OR K = (NcY * 3) + 1 OR K = NcY * 4 OR K = (NcY * 4) + 1 OR K = NcY * 5) THEN
                  'Laterales izq. y derecho
                   X1 = Cte7 + Cte3 - L7(K) * U7(K - 6) - L6(K) * U6(K - 5) - L5(K) * U5(K - 4)
                   X2 = -L4(K) * U4(K - 3) - L3(K) * U3(K - 2) - L2(K) * U2(K - 1)
                   L1(K) = X1 + X2
               ELSE
                   X1 = Cte7 - L7(K) * U7(K - 6) - L6(K) * U6(K - 5) - L5(K) * U5(K - 4)
                   X2 = -L4(K) * U4(K - 3) - L3(K) * U3(K - 2) - L2(K) * U2(K - 1)
                   L1(K) = X1 + X2
               END IF
             
            END IF
         
         END IF
        
         IF K <= NN - 1 THEN
            'Flanco derecho
             IF (K = NcY OR K = NcY * 2 OR K = NcY * 3 OR K = NcY * 4 OR K = NcY * 5) THEN
                X1 = -L6(K) * U7(K - 5) - L5(K) * U6(K - 4) - L4(K) * U5(K - 3)
                X2 = -L3(K) * U4(K - 2) - L2(K) * U3(K - 1)
                U2(K) = (X1 + X2) / L1(K)
             ELSE
                X1 = Cte3 - L6(K) * U7(K - 5) - L5(K) * U6(K - 4) - L4(K) * U5(K - 3)
                X2 = -L3(K) * U4(K - 2) - L2(K) * U3(K - 1)
                U2(K) = (X1 + X2) / L1(K)
             END IF
         END IF
        
         IF K >= 2 THEN
            'Flanco izq.
             IF (K = NcY + 1 OR K = (NcY * 2) + 1 OR K = (NcY * 3) + 1 OR K = (NcY * 4) + 1 OR K = (NcY * 5) + 1) THEN
                X1 = -L7(K) * U6(K - 6) - L6(K) * U5(K - 5) - L5(K) * U4(K - 4)
                X2 = -L4(K) * U3(K - 3) - L3(K) * U2(K - 2)
                L2(K) = X1 + X2
             ELSE
                X1 = Cte3 - L7(K) * U6(K - 6) - L6(K) * U5(K - 5) - L5(K) * U4(K - 4)
                X2 = -L4(K) * U3(K - 3) - L3(K) * U2(K - 2)
                L2(K) = X1 + X2
             END IF
         END IF
        
         IF K <= NN - 2 THEN
            X1 = -L5(K) * U7(K - 4) - L4(K) * U6(K - 3) - L3(K) * U5(K - 2) - L2(K) * U4(K - 1)
            U3(K) = X1 / L1(K)
         END IF
        
         IF K >= NcY - 3 THEN
            L3(K) = -L7(K) * U5(K - 6) - L6(K) * U4(K - 5) - L5(K) * U3(K - 4) - L4(K) * U2(K - 3)
         END IF
        
         IF K <= NN - 3 THEN
            U4(K) = (-L4(K) * U7(K - 3) - L3(K) * U6(K - 2) - L2(K) * U5(K - 1)) / L1(K)
         END IF
        
         IF K >= NcY - 2 THEN L4(K) = -L7(K) * U4(K - 6) - L6(K) * U3(K - 5) - L5(K) * U2(K - 4)
        
         IF K <= NN - 4 THEN U5(K) = (-L3(K) * U7(K - 2) - L2(K) * U6(K - 1)) / L1(K)
        
         IF K >= NcY - 1 THEN L5(K) = -L7(K) * U3(K - 6) - L6(K) * U2(K - 5)
        
         IF K <= NN - 5 THEN U6(K) = -L2(K) * U7(K - 1) / L1(K)
        
         IF K >= NcY THEN L6(K) = -L7(K) * U2(K - 6)
        
         IF K <= NN - NcY THEN U7(K) = Cte1 / L1(K)
        
         IF K >= NcY + 1 THEN L7(K) = Cte1
    
     NEXT K

END SUB

SUB matini
 
  'Subrutina que genera la matriz de coeficientes inicial
   
    'Inicializando los elementos de las vectores de presin y gasto
      FOR T = 0 TO Taux - 1
          FOR j = 1 TO NcY
              FOR i = 1 TO NcX
                  P(i, j, 0) = Pi
                  Q(i, j, T) = 0
      NEXT i, j, T
   
    'Inicializando los elementos de las matrices L y U
      FOR K = -10 TO (NN + 10)
          U2(K) = 0
          U3(K) = 0
          U4(K) = 0
          U5(K) = 0
          U6(K) = 0
          U7(K) = 0
          L1(K) = 0
          L2(K) = 0
          L3(K) = 0
          L4(K) = 0
          L5(K) = 0
          L6(K) = 0
          L7(K) = 0
      NEXT

END SUB

SUB paginacion (v)
 
  'Subrutina que controla la paginacin; si v = 1 cambia a la pgina que
  'muestra los datos de archivo, y si v > 1, despliegan las pantallas de
  'resultados, para cada incremento de tiempo (v)
   
    SELECT CASE v
           CASE 1
                'Se llama a la subrutina que despliega los datos de archivo
                  CALL imparch1
          
           CASE 2 TO Taux
                'Se invoca a la subrutina que crea un malla
                  CALL malla
               
                'Llamado de la subrutina que muestra resultados para cada
                'incremento de tiempo
                  CALL impsolu(v)
    END SELECT

END SUB

SUB recuadro
 
  'Subrutina que genera el recuadro principal y la barra inferior de
  'alternativas
   
   'Se declara la pantalla de trabajo, adems, se crean las lneas que
   'constituyen al recuadro y se le asigna color tanto a la pantalla como a
   'las mismas lneas
     VIEW PRINT
     COLOR 14, 3, 3
     CLS
    
     'Se conforma el recuadro
       PRINT CHR$(218);
       FOR X = 1 TO 23
           PRINT CHR$(196);
       NEXT
    
       COLOR 0, 7
         PRINT "SIMULADOR BIDIRECCIONAL DE FLUJO";
       COLOR 14, 3, 3
    
       FOR X = 1 TO 23
           PRINT CHR$(196);
       NEXT
    
       PRINT CHR$(191)
       LOCATE 23, 1: PRINT CHR$(192);
     
       FOR X = 1 TO 78
           PRINT CHR$(196);
       NEXT
    
       PRINT TAB(80); CHR$(217)
    
     'Despliegue de lnea de estado (opciones de trabajo)
       COLOR 15, 5
         LOCATE 24, 1
         PRINT "<Page Up = Pg. sig.>    <F1 = Ejecutar>   <Page Down = Pg. ant.>   <Esc = Fin>"
       COLOR 14, 3, 3
    
     'Se designa el rea de desplegado para textos y se trazan las lneas
     'verticales del marco
        VIEW PRINT 2 TO 23
        FOR X = 1 TO 21
            PRINT TAB(1); CHR$(179);
            PRINT TAB(80); CHR$(179)
        NEXT

END SUB

FUNCTION rotar (lower, upper, current, inc)
 
  'Funcin que regula los desplazamientos, del cursor, en sentido vertical
  'a travs de los campos, y de las pantallas, anterior o posterior; con un
  'incremento (inc)
  
   current = current + inc
   IF current > upper THEN current = lower
   IF current < lower THEN current = upper
   rotar = current

END FUNCTION

SUB solucion
 
  'Se genera la solucin del sistema
 
    'Se verifica que las celdas con pozo productor no hallan alcanzado la Pb,
    'en caso afirmativo, se simula el cierre el pozo y Q=0
    FOR T = 0 TO Taux - 1
       
        FOR KK = 1 TO Np
            FOR coorX = 1 TO NcX
                FOR coorY = 1 TO NcY
                    IF P(coorX, coorY, T) > Pb THEN
                       Q(coorX, coorY, T) = Rte(coorX, coorY)
                    ELSE
                       Q(coorX, coorY, T) = 0
                    END IF
        NEXT coorY, coorX, KK
       
        'Se genera el vector de trminos independientes
          K = 0
          FOR i = 1 TO NcX
              FOR j = 1 TO NcY
                  K = K + 1
                  TI(K) = Cte5 * P(i, j, T) + Cte6 * Q(i, j, T)
          NEXT j, i
       
        'Con el vector de trms. indeps. y la matriz generados en la
        'subrutina matgen, se da solucin al sistema
         
        'Inicializando vectores
          FOR K = -10 TO (NN + 10)
              y(K) = 0
              X(K) = 0
          NEXT
       
        'Sustitucin hacia adelante (matriz tringular superior)
          FOR K = 1 TO NN
              X1 = TI(K) - L2(K) * y(K - 1) - L3(K) * y(K - 2) - L4(K) * y(K - 3)
              X2 = -L5(K) * y(K - 4) - L6(K) * y(K - 5) - L7(K) * y(K - 6)
              y(K) = (X1 + X2) / L1(K)
          NEXT
       
        'Sustitucin hacia atrs
          FOR K = NN TO 1 STEP -1
              X1 = y(K) - U2(K) * X(K + 1) - U3(K) * X(K + 2) - U4(K) * X(K + 3)
              X2 = -U5(K) * X(K + 4) - U6(K) * X(K + 5) - U7(K) * X(K + 6)
              X(K) = X1 + X2
          NEXT
       
        'Cambio de variable en el vector solucin
          K = 0
          FOR i = 1 TO NcX
              FOR j = 1 TO NcY
                  K = K + 1
                  P(i, j, T + 1) = X(K)
          NEXT j, i
   
    NEXT T

END SUB

SUB torre
 
  'Subprograma que despliega en pantalla una torre de terminacin
 
    'Tubera
    COLOR 15, 3
      LOCATE 2, 40: PRINT ""
    COLOR 0, 3
 
    'Estructrura de la torre
    PRINT TAB(35); ""
    PRINT TAB(35); "     "
    PRINT TAB(35); "   "
    PRINT TAB(35); "  "
    PRINT TAB(35); "    "
    PRINT TAB(35); "     "
    PRINT TAB(35); "   "
    PRINT TAB(35); "      "
   PRINT TAB(34); "   "
   PRINT TAB(34); "     "
   PRINT TAB(34); "         "
 PRINT TAB(32); " "
 PRINT TAB(32); "  Ĵ"
 PRINT TAB(32); "  Ĵ"

    'Estratos
    COLOR 6, 1
      LOCATE 17, 1: PRINT STRING$(80, 176)
      LOCATE 19, 1: PRINT STRING$(80, 177)

    'Tubera
    COLOR 15, 3
      FOR i = 4 TO 13
          LOCATE i, 40: PRINT ""
      NEXT
   
    COLOR 15, 0
      FOR i = 15 TO 21
          LOCATE i, 40: PRINT ""
      NEXT
     
      'Tubera disparada
      LOCATE 22, 40: PRINT ""
      LOCATE 23, 40: PRINT ""
    COLOR 0, 3

    'Estructrura de la torre
    LOCATE 10, 40: PRINT ""
    LOCATE 14, 40: PRINT ""

END SUB

