
   'Programa fuente para interpolacin en tres dimensiones mediante la
   'subrutina SPLINE y la funcin SEVAL.
  
   'Datos tomados del libro de apuntes de Computacin Aplicada a la
   'Ingeniera Petrolera.
           
       'Versin 1.0 (Programa diseado por J. Antonio Ruiz G., 1991)
    
    'LISTA DE LAS PRINCIPALES VARIABLES EMPLEADAS EN EL PROGRAMA. 
   
       'SIMBOLO EN EL PROGRAMA                  DESCRIPCION
        
        'B(I), C(I), D(I),
        'Baux(I), Caux(I)
        'y Daux(I) ............. Vectores de coeficientes de las funciones
                                 'cbicas calculadas por SPLINE.
        'M ..................... Nmero de datos de temperatura (ordenadas).
        'N ..................... Nmero de datos de presin (abscisas).
        'OP .................... Variable que determina si la bsqueda del
                                 'intervalo apropiado, dentro de la funcin
                                 'SEVAL, ser binaria o secuencial.
        'X(I) .................. Vector de abscisas en estricto orden ascen-
                                 'dente (presin), [lb/pg abs].
        'Xint .................. Abscisa a interpolar (presin), [lb/pg abs].
        'Y(J) .................. Vector de ordenadas (temperatura), [70F].
        'Yint .................. Ordenada a interpolar (temperatura), [70F].
        'Z(I) .................. Vector de cotas (factor de compresibilidad).
        'Zaux(J) ............... Vector de funciones cbicas evaluadas en el,
                                 'punto Xint(K).
        'Zint .................. Valor interpolado del factor de compresibi-
                                 'lidad, correspondiente a Xint(K) y Yint(K).
    
     DEFINT I-K, M-N
     DECLARE SUB apuntador (FIRST%, LAST%, DX%, Y%, FLAG1%)
     DECLARE SUB archicrea ()
     DECLARE SUB captura (letrero$, var!, min!, max!, ren%, col%, colcap%)
     DECLARE SUB marco (reni%, renf%, coli%, colf%, tipo%, ci%, cf%)
     DECLARE SUB opcion1 ()
     DECLARE SUB opcion2 ()
     DECLARE SUB opcion3 ()
     DECLARE SUB opciones ()
     DECLARE FUNCTION seval (N, U, X(), Y(), B(), c(), D(), OP)
     DECLARE SUB spline (N, X(), Y(), B(), c(), D())
     DECLARE SUB torre ()
    
     'Declaracin de constantes
       CONST enter = 13
    
     'Se define el rea visual de trabajo y se le asigna color
       VIEW PRINT 1 TO 25
       COLOR 15, 3, 3
       CLS
      
     BEEP
     CLEAR
    
     'Proteccin contra errores durante la ejecucin del programa
       ON ERROR GOTO 10
    
     'Se invoca a la subrutina que controla las opciones de trabajo
       CALL opciones

     'Seccin de deteccin de errores
10     SOUND 720, 10
       CLS
       COLOR 31
         LOCATE 12, 12
         PRINT "Error detectado: Revise la informacin, verifique la"
         LOCATE 13, 29: PRINT "existencia de las bases de  datos o"
         LOCATE 14, 29: PRINT "corrija el patrn de bsqueda."
       COLOR 3
       RESUME 20

20    END

DEFSNG I-K, M-N
SUB apuntador (FIRST%, LAST%, DX%, Y%, FLAG1%)
 
 'Subprograma para control de seleccin de opciones en pantalla
 
   'ENTRADA:
      'FIRST% = Valor mnimo o inicial de la coord. X del cursor (rengln),
               'el valor asciende de la parte sup. a la inferior del  moni-
               'tor.
      ' LAST% = Valor mximo o final de la coord. X del  cursor  (rengln),
               'el valor se incrementa de la parte izq. a  la  derecha  del
               'monitor.
           'NOTA: [LAST% - FIRST%] + 1 = nm. de alternativas de trabajo.
      '   DX% = Valor que modifica la posicin inicial del cursor.
      '    X% = Coord. X donde aparecera el cursor { FIRST% + DX% }
      '    Y% = Coord. Y (ordenada o columna) donde aparecera el cursor.
 
   'SALIDA:
      'FLAG1% = Valor nulo en la entrada y de la alternativa en la salida.
  
  '
    
     'Coordenada X del cursor
       X% = FIRST% + DX%
   
     'Se define visibilidad, tamao y posicin del cursor
       LOCATE X%, Y%, 1, 0, 7

     'Se asigna el valor de una tecla a una variable alfanumrica
a10:   ANS$ = INKEY$
       IF ANS$ = "" THEN GOTO a10
     
       'Se teclea la flecha de desplazamiento hacia bajo
       IF ANS$ = CHR$(0) + CHR$(80) THEN
          'Nueva coordenada X (rengln)
          IF X% < LAST% THEN
             'Incremento
             X% = X% + 1
          ELSE
             'Posicin inicial
             X% = FIRST%
          END IF
       END IF
     
       'Se teclea la flecha de desplazamiento hacia arriba
       IF ANS$ = CHR$(0) + CHR$(72) THEN
          'Nueva coordenada X (rengln)
          IF X% > FIRST% THEN
             'Decremento
             X% = X% - 1
          ELSE
             'Posicin final
             X% = LAST%
          END IF
       END IF
     
     'Se despliega el cursor en las coordenadas actuales
       LOCATE X%, Y%
               
     'Si no se oprime < ENTER > se contina el proceso
       IF ANS$ <> CHR$(13) THEN GOTO a10
   
     'Se asigna a la variable FLAG% el valor de la alternativa elegida en
     'funcin de su posicin
       FLAG1% = CSRLIN
       FLAG1% = FLAG1% - (FIRST% - 1)
   
     'Se redefine la visibilidad y el tamao del cursor
       LOCATE , , 0, 5, 6

END SUB

SUB captura (letrero$, var!, min!, max!, ren%, col%, colcap%)
 
  'Subprograma que controla la captura de informacin de la variable "var"
   
    'letrero$ -- Frase a desplegar.
    'var      -- Nombre de la variable de entrada.
    'min, max -- Valores mn. y mx..
    'ren,col  -- Rengln y columna de desplegado.
    'colcap   -- Columna de captura y de borrado en caso de error.
   
    DO
      LOCATE ren%, col%: PRINT letrero$; : INPUT var
      'Se verifica que el dato introducido est dentro de un rango lgico
      IF var < min! OR var > max! THEN
         SOUND 300, 3: LOCATE ren%, colcap%: PRINT "          "
      END IF
    LOOP WHILE var < min! OR var > max!

END SUB

SUB marco (reni%, renf%, coli%, colf%, tipo%, ci%, cf%)

 'Subprograma que genera un marco
  
   'reni%, renf%-- Rengln inicial y final del recuadro
   'coli%, colf%-- Columna inicial y final del recuadro
   'tipo%       -- 1 para lnea sencilla, 2 para doble y 3 para semi-gruesa
   'ci%, cf%    -- Color inicial (recuadro) y final
  
   'Caractersticas del marco (tipo de lnea)
   SELECT CASE tipo%
          CASE 1
               'lnea sencilla
               esi$ = "": lhs$ = "": lv$ = "": esd$ = "": eii$ = ""
               lhi$ = "": eid$ = ""
          CASE 2
               'lnea doble
               esi$ = "": lhs$ = "": lv$ = "": esd$ = "": eii$ = ""
               lhi$ = "": eid$ = ""
          CASE 3
               'lnea semi-gruesa
               esi$ = "": lhs$ = "": lv$ = "": esd$ = "": eii$ = ""
               lhi$ = "": eid$ = ""
   END SELECT
  
   'Se despliega el recuadro
   COLOR ci%
     LOCATE reni%, coli%: PRINT esi$
     LOCATE reni%, colf%: PRINT esd$
     LOCATE reni%, coli% + 1: PRINT STRING$(colf% - coli% - 1, lhs$)
     FOR I% = reni% + 1 TO renf%
         LOCATE I%, coli%: PRINT lv$
         LOCATE I%, colf%: PRINT lv$
     NEXT I%
     LOCATE renf%, coli%: PRINT eii$
     LOCATE renf%, colf%: PRINT eid$
     LOCATE renf%, coli% + 1: PRINT STRING$(colf% - coli% - 1, lhi$)
   COLOR cf%

END SUB

DEFINT I-K, M-N
SUB opcion1
  
  'Subprograma que efecta la interpolacin en tres dimensiones
    '  X = Abscisas en extricto orden ascendente (presin).
    '  Y = Ordenadas (temperaturas).
    '  Z = Cotas (factor de compresibilidad).
   
    'Se redefine la visibilidad y el tamao del cursor
      LOCATE , , 0, 5, 6
   
    'Invocando a la subrutina que genera un marco
      CALL marco(1, 23, 2, 79, 3, 5, 15)
      CALL marco(3, 6, 10, 70, 1, 1, 15)
     
      COLOR 15
        LOCATE 4, 12: PRINT "INTERPOLACION SECUENCIAL O BINARIA, EN TRES DIMENSIONES,"
        LOCATE 5, 22: PRINT "USANDO LAS SUBRUTINAS SPLINE Y SEVAL"
      COLOR 8
        LOCATE 8, 16: PRINT "Valores mximos: P = 6.0 lb/pgabs y T = 300F"
        LOCATE 9, 16: PRINT "Valores mnimos: P = 0.5 lb/pgabs y T =   0F"
      
    'Nm. datos presin (N) y nm. datos temp.(M)
      N = 7
      M = 4
   
    'Dimensionando vectores y matrices involucrados
      DIM X(N), Y(M), B(N), c(N), D(N), Z(N), Baux(M), Caux(M), Daux(M), Zaux(M)
   
    'Se abre el archivo #1 (pres.dat para lectura de presiones), en el
    'patrn de bsqueda especificado
      OPEN "a:\cap3\pres.dat" FOR INPUT AS #1
      FOR I = 1 TO N
          INPUT #1, X(I)
      NEXT
   
    'Se cierra el archivo #1
      CLOSE #1
   
    DO
       COLOR 0
       'Se realiza la captura de valores a interpolar
         CALL captura("Presin a interpolar [lb/pgabs] ", Xint, .5, 6, 13, 22, 57)
         CALL captura("Temperatura a interpolar [F] ", Yint, 0, 300, 14, 22, 54)
      
       'Se abre una base de datos para cada temperatura
       FOR J = 1 TO M
           'Para cada temperatura existe una base de datos
           IF J = 1 THEN temp$ = "32"
           IF J = 2 THEN temp$ = "100"
           IF J = 3 THEN temp$ = "190"
           IF J = 4 THEN temp$ = "280"
           nombre$ = "a:\cap3\T-" + temp$ + ".DAT"
          
           'Se abre el archivo nombre$ para lectura de factores de compresi-
           'bilidad y de temperatura
             OPEN nombre$ FOR INPUT AS #1
             INPUT #1, Y(J)
             FOR K = 1 TO N
                 INPUT #1, Z(K)
             NEXT
             CLOSE #1
          
           'Se invoca a la subrutina SPLINE para los valores de X y Z
             CALL spline(N, X(), Z(), B(), c(), D())
          
           'Se invoca la funcin SEVAL para los valores de X, Z y Xint
             Zaux(J) = seval(N, Xint, X(), Z(), B(), c(), D(), 0)
       NEXT J
      
       'Se invoca a la subrutina SPLINE para los valores de Y y Z
         CALL spline(M, Y(), Zaux(), Baux(), Caux(), Daux())
      
       'Se invoca la funcin SEVAL para los valores de Y, Z y Yint
         Zint = seval(M, Yint, Y(), Zaux(), Baux(), Caux(), Daux(), 0)
      
       'Zint es el valor interpolado de la ordenada
         LOCATE 16, 26
         PRINT "Z interpolada = "; : PRINT USING "#.####"; Zint
       
        'Continuar o terminar
          COLOR 31
            LOCATE 20, 21: PRINT "< ENTER > continuar, < ESC > terminar"
          COLOR 0
         
          DO
            K$ = INKEY$
          LOOP WHILE K$ = ""
         
          'Transmutacin a cdigo ASCII
            qy = ASC(RIGHT$(K$, 1))
         
          FOR I = 1 TO 8
            LOCATE 12 + I, 20: PRINT "                                                   "
          NEXT
    
     LOOP UNTIL qy <> enter
    
     SOUND 200, 2

END SUB

DEFSNG I-K, M-N
SUB opcion2
 
  'Subprograma para el control de la opcin 2 del men principal
    
   LOCATE 3, , 0
   PRINT TAB(4); "DATOS (tomados del libro de apuntes de Comput. Aplic. a la Ingra. Pet.):"
   PRINT
   PRINT TAB(6); "El presente estudio, corresponde a un gas natural que contiene 0.7% de"
   PRINT TAB(6); "nitrgeno. Mediante un anlisis de laboratorio se midi su  factor  de"
   PRINT TAB(6); "compresibilidad (Z), para un rango de presin de 1.4 a 5.0 lb/pg, y -"
   PRINT TAB(6); "para temperatura, de 32 a 280F. La tabla siguiente  muestra  los  re-"
   PRINT TAB(6); "sultados obtenidos, los cuales deben implementarse en  este  programa,"
   PRINT TAB(6); "al crear las bases de datos por vez primera."
  
   LOCATE 13, , 0: PRINT TAB(14); "Presin                        Temperatura"
   PRINT TAB(12); "[lb/pgabs]      32[F]     100[F]     190[F]     280[F]"
   PRINT TAB(10); : PRINT STRING$(63, "-")
   PRINT TAB(16); "1.4          0.6885     0.8213      0.9097      0.9557"
   PRINT TAB(16); "1.6          0.6593     0.8044      0.9009      0.9516"
   PRINT TAB(16); "1.8          0.6433     0.7896      0.8943      0.9486"
   PRINT TAB(16); "2.0          0.6369     0.7805      0.8899      0.9472"
   PRINT TAB(16); "3.0          0.6981     0.7901      0.8932      0.9571"
   PRINT TAB(16); "4.0          0.8103     0.8600      0.9356      0.9890"
   PRINT TAB(16); "5.0          0.9333     0.9516      1.0050      1.0384"
  
   CALL marco(1, 24, 2, 79, 2, 1, 0)
    
     COLOR 31
       LOCATE 24, 30: PRINT " < Para continuar presione cualquier tecla > "
     COLOR 0
    
     DO
     LOOP WHILE INKEY$ = ""

END SUB

SUB opcion3
 
  'Subrutina que crea las bases de datos para la presin, la temperatura
  'y el factor de compresibilidad
  
   'Nm. de datos de presin (N) (eje X) y nm. de datos de temp. (M) (eje Y)
     N = 7
     M = 4
  
   COLOR 15
     LOCATE 7, 7
     PRINT "Nota: archivos diseados para 7 datos de presin y 4 de temperatura."
  
   CALL marco(8, 20, 11, 70, 2, 1, 0)
  
   LOCATE 9, 25: PRINT "Creando el archivo:  pres.dat"
  
   'Se crea el archivo pres.dat
     OPEN "a:\cap3\pres.dat" FOR OUTPUT AS #1
     'Captura de informacin
     FOR I = 1 TO N
         COLOR 8
           LOCATE 10 + I, 22
           PRINT "Presin "; I; " [lb/pgabs]"; : INPUT P: PRINT #1, P
     NEXT
  
   'Se cierra el archivo
     CLOSE #1
  
   CLS
   COLOR 15
     LOCATE 3, 7
     PRINT "Nota: archivos diseados para 7 datos de presin y 4 de temperatura."
  
   CALL marco(1, 23, 2, 79, 3, 14, 0)
  
   'Captura de informacin
   FOR J = 1 TO M
       LOCATE 5, 15
       PRINT "Temperatura "; J; " [F] "; : INPUT temp$
       nombre$ = "T-" + temp$ + ".DAT"
      
       CALL marco(8, 20, 11, 70, 2, 1, 0)
      
       LOCATE 9, 25
       PRINT "Creando el archivo:  "; nombre$
      
       'Se crea el archivo nombre$ (en el patrn especificado), para tempera-
       'tura y factor de compresibilidad
         nombre$ = "a:\cap3\" + nombre$
         OPEN nombre$ FOR OUTPUT AS #1
         PRINT #1, temp$
        
         FOR K = 1 TO N
             COLOR 8
               LOCATE 10 + K, 20
               PRINT "Factor de comp."; K; " @ Tant. y P"; K; : INPUT Z
               PRINT #1, Z
         NEXT
        
         'Se cierra el archivo
         CLOSE #1
        
         CLS
      
       COLOR 15
         LOCATE 3, 7
         PRINT "Nota: archivos diseados para 7 datos de presin y 4 de temperatura."
      
       CALL marco(1, 23, 2, 79, 3, 14, 0)
  
   NEXT J

END SUB

SUB opciones

 'Subrutina que facilita una seleccin de alternativas con el cursor como
 'apuntador

   'Se invoca a la subrutina que genera un marco
     CALL marco(9, 15, 9, 72, 3, 5, 0)
     CALL marco(10, 14, 11, 70, 1, 1, 15)
  
   LOCATE 11, 18: PRINT "Interpolacin del factor de compresiblidad (Z)"
   LOCATE 12, 19: PRINT "para un gas que contiene 0.7 % de nitrgeno,"
   LOCATE 13, 17: PRINT "empleando la subrutina SPLINE y la funcin SEVAL"
  
   COLOR 31
     LOCATE 23, 19: PRINT "< Para continuar presione cualquier tecla >"
   COLOR 0
   DO
   LOOP WHILE INKEY$ = ""
 
  '-------------- Despliegue grfico de opciones maestras -------------------

p10: CLS
    'Se invoca a la subrutina que genera un marco
      CALL marco(8, 15, 11, 68, 3, 0, 15)
   
    LOCATE 8, 32:  PRINT " MENU PRINCIPAL "
    LOCATE 10, 16: PRINT "_ Interpolacin del factor de compresibilidad (Z)."
    LOCATE 11, 16: PRINT "_ Visualizar los datos bsicos del gas en estudio."
    LOCATE 12, 16: PRINT "_ Crear las bases de datos (instalacin)."
    LOCATE 13, 16: PRINT "_ Trmino de la sesin."
   
    COLOR 16
      LOCATE 20, 20: PRINT "    Seleccione la opcin de trabajo     "
      LOCATE 21, 20: PRINT "        y   presione  < ENTER >         "
    COLOR 0
   
    'Control de seleccin de opciones en pantalla
      CALL apuntador(10, 13, 0, 16, FLAG1%)
      SOUND 450, 4
   
    'Designacin de alternativa
      ON FLAG1% GOSUB p20, p30, p40, p50

p20: 'Opcin 1 (interpolacin del factor de compresibilidad)
      CLS
       'Llamada a la subrutina de interpolacin
         CALL opcion1
         GOTO p10

p30: 'Opcin 2 (visualizar los datos bsicos del gas en estudio)
      CLS
       'Se invoca a la subrutina de desplegado de informacin
         CALL opcion2
         GOTO p10

p40: 'Opcin 3 (crear las bases de datos)
      CLS
       LOCATE 3, 6, 0
       PRINT "ESTA OPCION SOLO DEBE USARSE AL EJECUTAR EL PROGRAMA POR VEZ PRIMERA,"
       PRINT TAB(6); "DE LO CONTRARIO, SE CORRE EL RIESGO DE ELIMINAR LAS  BASES  DE  DATOS"
       PRINT TAB(6); "PREVIAMENTE CREADAS."
      
       'Se invoca a la subrutina que genera un marco
        CALL marco(1, 23, 2, 79, 3, 14, 0)
      
       'Se ofrece la alternativa de abortar la instalacin
        COLOR 31
          LOCATE 10, 21: PRINT "< ENTER > continuar, < ESC > terminar"
        COLOR 0
        DO
          K$ = INKEY$
        LOOP WHILE K$ = ""
       
        'Transmutacin a cdigo ASCII
        qy = ASC(RIGHT$(K$, 1))
        IF qy <> enter THEN GOTO p10
        LOCATE 10, 21: PRINT "                                        "
       
        'Llamado a la subrutina que crea las bases de datos
          CALL opcion3
        GOTO p10

p50: 'Opcin 4
      CLS
      
       'Se invoca a la subrutina que genera un marco
        CALL marco(22, 24, 52, 70, 3, 17, 0)
      
       'Se invoca la subrutina que despliega una torre
        CALL torre
      
       COLOR 31, 3
         LOCATE 23, 54
         PRINT " FIN DE SESION "
       COLOR 3, 3, 3
     
       'Se redefinen las caractersticas del cursor
        LOCATE , , 0
     
       END

END SUB

DEFINT I-K, M-N
FUNCTION seval (N, U, X(), Y(), B(), c(), D(), OP)
  
   'Esta subrutina evala la funcin cbica spline:
  
      'seval = Y(I) + B(I)*(U-X(I)) + C(I)*(U-X(I))^2 + D(I)*(U-X(I))^3
  
   'donde:  X(I) < U < X(I+1), usando la regla de Horner.
  
   'Si U < X(1) entoces se supone que I=1.
   'Si U > X(N) entoces se supone que I=N.
  
   'Datos de entrada:
          'N = Nmero de datos.
          'U = Abscisa en la que se valuar el spline.
       'X, Y = Vectores de abscisas y ordenadas dato.
    'B, C, D = Vectores de coeficientes clculados por SPLINE.
         'OP = Variable que rige el tipo de bsqueda, secuencial si OP=1
              'y binaria en cualquier otro caso.
  
   'Si U no se ubica en el intervalo de la llamada previa, entonces una
   'bsqueda binaria o secuencial ser ejecutada,  para  determinar  el
   'intervalo apropiado.
   
   I = 1
   IF I >= N THEN I = 1
  
   'Se designa el mtodo de bsqueda
     SELECT CASE OP
           
            CASE 1
                 IF U <= X(I + 1) THEN GOTO se10
                 'Bsqueda secuencial para U < X(I)
                   K = 0
                   DO
                     K = K + 1
                     IF U < X(K) THEN J = K
                     IF U >= X(K) THEN I = K
                   LOOP WHILE K < N
           
            CASE ELSE
                 IF U <= X(I + 1) THEN GOTO se10
                 'Bsqueda binaria para U < X(I)
                   I = 1
                   J = N + 1
                   DO
                     K = INT((I + J) / 2)
                     IF U < X(K) THEN J = K
                     IF U >= X(K) THEN I = K
                   LOOP WHILE J > I + 1
    
     END SELECT
  
   'Se evala el spline
se10: DX = U - X(I)
      seval = Y(I) + DX * (B(I) + DX * (c(I) + DX * D(I)))

END FUNCTION

DEFSNG J
SUB spline (N, X(), Y(), B(), c(), D())
  
   'Los coeficientes B(I), C(I) y D(I), I=1,2,...,N, son calculados para
   'formar el polinomio de Splines cbicos:
  
     'S(X) = Y(I) + B(I)*(X-X(I)) + C(I)*(X-X(I))^2 + D(I)*(X-X(I))^3
  
   'para: X(I) =< X =< X(I+1).
  
   'Datos de entrada:
     'N = Nmero de datos, puntos o coordenadas (N >= 2).
     'X = Abscisa de las coordenadas dato en estricto orden ascendente.
     'Y = Ordenada de las coordenadas dato.
  
   'Datos de salida:
     'B, C, D = Vectores con coeficientes de SPLINE, indicados anteriormente.
  
   'Usando la letra "P" para denotar diferenciales:
     'Y(I) = S(X(I))
     'B(I) = SP(X(I))
     'C(I) = SPP(X(I))/2
     'D(I) = SPPP(X(I))/6
  
   'Nota: La funcin SEVAL evala el spline.
  
     NM1 = N - 1
     IF N < 2 THEN
        SOUND 200, 4: CLS
        COLOR 31
          LOCATE 12, 22: PRINT "EL No. DE DATOS DEBE SER MAYOR DE 2 !!"
        COLOR 7
        END
     END IF
     IF N < 3 THEN
        B(1) = (Y(2) - Y(1)) / (X(2) - X(1))
        c(1) = 0!
        D(1) = 0!
        B(2) = B(1)
        c(2) = 0!
        D(2) = 0!
        EXIT SUB
     END IF
  
   'Se crea un sistema tri-diagonal
   'B=diagonal principal, D=diagonal inmediata-superior, C=vector de trminos
   'independientes
     D(1) = X(2) - X(1)
     c(2) = (Y(2) - Y(1)) / D(1)
     FOR I = 2 TO NM1
         D(I) = X(I + 1) - X(I)
         B(I) = 2! * (D(I - 1) + D(I))
         c(I + 1) = (Y(I + 1) - Y(I)) / D(I)
         c(I) = c(I + 1) - c(I)
     NEXT I
  
   'Condiciones finales. La tercera derivada de X(1) y X(N) resultan del
   'cociente de las diferencias
     B(1) = -D(1)
     B(N) = -D(N - 1)
     c(1) = 0!
     c(N) = 0!
     IF N = 3 THEN GOTO sp10
     c(1) = c(3) / (X(4) - X(2)) - c(2) / (X(3) - X(1))
     c(N) = c(N - 1) / (X(N) - X(N - 2)) - c(N - 2) / (X(N - 1) - X(N - 3))
     c(1) = c(1) * D(1) ^ 2 / (X(4) - X(1))
     c(N) = -c(N) * D(N - 1) ^ 2 / (X(N) - X(N - 3))
  
   'Sustitucin hacia adelante
sp10: FOR I = 2 TO N
          T = D(I - 1) / B(I - 1)
          B(I) = B(I) - T * D(I - 1)
          c(I) = c(I) - T * c(I - 1)
      NEXT I
  
   'Sustitucin hacia atrs
     c(N) = c(N) / B(N)
     FOR IB = 1 TO NM1
         I = N - IB
         c(I) = (c(I) - D(I) * c(I + 1)) / B(I)
     NEXT IB
  
   'Clculo de los coeficientes del polinomio
     B(N) = (Y(N) - Y(NM1)) / D(NM1) + D(NM1) * (c(NM1) + 2! * c(N))
     FOR I = 1 TO NM1
         B(I) = (Y(I + 1) - Y(I)) / D(I) - D(I) * (c(I + 1) + 2! * c(I))
         D(I) = (c(I + 1) - c(I)) / D(I)
         c(I) = 3! * c(I)
     NEXT I
     c(N) = 3! * c(N)
     D(N) = D(N - 1)

END SUB

DEFSNG I, K, M-N
SUB torre
 
  'Subprograma que despliega en pantalla una torre de terminacin
   
    'Tubera
    COLOR 15, 3
      LOCATE 2, 40: PRINT ""
    COLOR 0, 3
    
    'Estructrura de la torre
    PRINT TAB(35); ""
    PRINT TAB(35); "     "
    PRINT TAB(35); "   "
    PRINT TAB(35); "  "
    PRINT TAB(35); "    "
    PRINT TAB(35); "     "
    PRINT TAB(35); "   "
    PRINT TAB(35); "      "
   PRINT TAB(34); "   "
   PRINT TAB(34); "     "
   PRINT TAB(34); "         "
 PRINT TAB(32); " "
 PRINT TAB(32); "  Ĵ"
 PRINT TAB(32); "  Ĵ"

    'Estratos
    COLOR 6, 1
    'COLOR 0, 6
      LOCATE 17, 1: PRINT STRING$(80, 176)
      LOCATE 19, 1: PRINT STRING$(80, 177)

    'Tubera
    COLOR 15, 3
      FOR I = 4 TO 13
          LOCATE I, 40: PRINT ""
      NEXT
  
    COLOR 15, 0
      FOR I = 15 TO 21
          LOCATE I, 40: PRINT ""
      NEXT
    
      'Tubera disparada
      LOCATE 22, 40: PRINT ""
      LOCATE 23, 40: PRINT ""
    COLOR 0, 3

    'Estructrura de la torre
    LOCATE 10, 40: PRINT ""
    LOCATE 14, 40: PRINT ""

END SUB

