
  'Programa que obtiene los coeficientes de la ec. para la optimizacin de
  'la perforacin propuesta en el artculo "A Multiple Regresion  Approach
  'to Optimal Drilling and Abnormal Pressure Detection",  de  Bourgoyne  y
  'Young, SPE, 1974.

	  'Versin 1.0 (Programa diseado por J. Antonio Ruiz G., 1991)

  'LISTA DE LAS PRINCIPALES VARIABLES EMPLEADAS EN EL PROGRAMA
 
    'SIMBOLO EN EL PROGRAMA                   DESCRIPCION

	 'A(I,J) ................... Matriz de dise7n1o para el ajuste.
	 'AltDientGastado(I) ....... Vector de altura del diente gastado,
						    '[fraccin].
	 'C(I) ..................... Vector de coeficientes obtenidos por la,
						    'subrutina SVD.
	 'Cond ..................... Nmero de condicin de la matriz de diseo.
	 'DensEquivCirc(I) ......... Vector de densidad equivalente, [lb/gal].
	 'GradPresForm(I) .......... Vector del gradiente de presin de forma-
						    'cin, [lb/gal].
	 'M ........................ Nmero de datos de perforacin.
	 'N ........................ Nmero de coeficientes del polinomio a
						    'ajustar (N = grado del polinomio + 1).
	 'NM ....................... Mximo dimensionamiento entre N y M.
	 'PesoSobreBarrena(I) ...... Vector de peso sobre la barrena,
						    '[1000 lb/pg].
	 'Prof(I) .................. Vector de profundidad perforada, [pie].
	 'Reynolds(I) .............. Vector del nmero de Reynolds, [adimen-
						    'sional].
	 'U(I,J), V(I,J),
	 'SIGMA(I), WORK(I) ........ Arreglos usados por la subrutina SVD.
	 'VelPenetrac(I) ........... Vector de velocidad de penetracin,
						    '[pie/hr].
	 'VelRotac(I) .............. Vector de velocidad de la rotaria, [rpm].
	 'Wdt ...................... Peso inicial sobre la barrena, [1000 lb/pg].
	 'Y(I) ..................... Vector de ordenadas dato.
   
    DECLARE SUB apuntador (FIRST%, LAST%, DX%, Y%, FLAG1%)
    DECLARE SUB archi (selec!)
    DECLARE SUB coeficientes ()
    DECLARE SUB marco (reni%, renf%, coli%, colf%, tipo%, ci%, cf%)
    DECLARE SUB datos ()
    DECLARE SUB op1 ()
    DECLARE SUB op2 ()
    DECLARE SUB op3 ()
    DECLARE SUB opciones ()
    DECLARE SUB pausa (ren%, col%, ci%, cf%)
    DECLARE SUB svd (NM!, M!, N!, A!(), W!(), MATU!, U!(), MATV!, V!(), IERR!, RV1!())
    DECLARE SUB tabla (T1!(), T2!(), T3!(), T4!(), T5!(), T6!(), T7!(), T8!())
    DECLARE SUB torre ()
   
    'Variables comunes a las subrutinas
	 COMMON SHARED N, M, NM, Wdt
   
    'Se designa el rea visual de trabajo y se le asigna color
	 VIEW PRINT
	 COLOR 0, 1, 1
	
	 CLEAR
	 CLS
	 BEEP
   
    'Proteccin contra errores durante la ejecucin del programa
	 ON ERROR GOTO 10
   
    'Declaracin de constantes
	 CONST enter = 13
   
    'Nmero de renglones de la matriz de diseo (M) (Nm. de datos),
    'nmero de columnas de la matriz de diseo (N=grado del polinomio + 1)
    'y mximo dimensionamiento entre N y M
	 M = 30
	 N = 8
	 NM = M
   
    'Dimensionamiento dinmico y comn de los arreglos que se utilizarn
	 DIM SHARED A(M, N), Y(M), C(M), SIGMA(M), U(M, M), V(M, M), RV1(M)
	 DIM SHARED Profun(M), GradPresForm(M), DensEquivCirc(M), PesoSobreBarrena(M)
	 DIM SHARED VelRotac(M), AltDientGastado(M), Reynolds(M), VelPenetrac(M)
   
    'Se invoca a la subrutina que controla las opciones de trabajo
	 CALL opciones
   
    'Seccin de deteccin de errores
10    SOUND 720, 10
	 CLS
	 COLOR 31
	   LOCATE 13, 12
	   PRINT "Error detectado: Revise los datos, verifique la existencia"
	   LOCATE 14, 12
	   PRINT "de las bases de datos o corrija el patrn de bsqueda."
	 COLOR 3
	 RESUME 20

20  END

SUB apuntador (FIRST%, LAST%, DX%, Y%, FLAG1%)
 
 'Subprograma para control de seleccin de opciones en pantalla

   'ENTRADA:
	 'FIRST% = Valor mnimo o inicial de la coord. X del cursor (rengln),
			'el valor asciende de la parte sup. a la inferior del  moni-
			'tor.
	 ' LAST% = Valor mximo o final de la coord. X del  cursor  (rengln),
			'el valor se incrementa de la parte izq. a  la  derecha  del
			'monitor.
		 'NOTA: [LAST% - FIRST%] + 1 = nm. de alternativas de trabajo.
	 '   DX% = Valor que modifica la posicin inicial del cursor.
	 '    X% = Coord. X donde aparecera el cursor { FIRST% + DX% }
	 '    Y% = Coord. Y (ordenada o columna) donde aparecera el cursor.

   'SALIDA:
	 'FLAG1% = Valor nulo en la entrada y de la alternativa en la salida.
 
  '
   
	'Coordenada X del cursor
	  X% = FIRST% + DX%
  
	'Se define visibilidad, tamao y posicin del cursor
	  LOCATE X%, Y%, 1, 0, 7

	'Se asigna el valor de una tecla a una variable alfanumrica
a10:   ANS$ = INKEY$
	  IF ANS$ = "" THEN GOTO a10
    
	  'Se teclea la flecha de desplazamiento hacia bajo
	  IF ANS$ = CHR$(0) + CHR$(80) THEN
		'Nueva coordenada X (rengln)
		IF X% < LAST% THEN
		   'Incremento
		   X% = X% + 1
		ELSE
		   'Posicin inicial
		   X% = FIRST%
		END IF
	  END IF
    
	  'Se teclea la flecha de desplazamiento hacia arriba
	  IF ANS$ = CHR$(0) + CHR$(72) THEN
		'Nueva coordenada X (rengln)
		IF X% > FIRST% THEN
		   'Decremento
		   X% = X% - 1
		ELSE
		   'Posicin final
		   X% = LAST%
		END IF
	  END IF
    
	'Se despliega el cursor en las coordenadas actuales
	  LOCATE X%, Y%
		    
	'Si no se oprime < ENTER > se contina el proceso
	  IF ANS$ <> CHR$(13) THEN GOTO a10
  
	'Se asigna a la variable FLAG% el valor de la alternativa elegida en
	'funcin de su posicin
	  FLAG1% = CSRLIN
	  FLAG1% = FLAG1% - (FIRST% - 1)
  
	'Se redefine la visibilidad y el tamao del cursor
	  LOCATE , , 0, 5, 6

END SUB

SUB archi (selec)
 
  'Subprograma para manejo de las bases de datos.
   
    'selec -- Define si se leer (selec = 0) o se escribir (selec <> 0)
		   'dicho archivo.
    
	'Se abre PROFUN.DTO para manejar datos de profundidad
	IF selec = 0 THEN
	   'Lectura de archivo
	   OPEN "A:\CAP3\profun.dto" FOR INPUT AS #1
	   FOR KK = 1 TO M
		  INPUT #1, Profun(KK)
	   NEXT
	ELSE
	   'Escritura de archivo
	   LOCATE 10, 45: PRINT "Profun.dto"
	   OPEN "A:\CAP3\profun.dto" FOR OUTPUT AS #1
	   FOR KK = 1 TO M
		  LOCATE 14, 20
		  PRINT "Profundidad ("; KK; ") [pie] = "; : INPUT "", Profun
		  LOCATE 14, 46
		  PRINT "                  ": PRINT #1, Profun
	   NEXT
	END IF
    
	'Se abre VELPENET.DTO para manejo de datos de vel. de penetracin
	IF selec = 0 THEN
	   'Lectura de archivo
	   OPEN "A:\CAP3\velpenet.dto" FOR INPUT AS #2
	   FOR KK = 1 TO M
		  INPUT #2, VelPenetrac(KK)
	   NEXT
	ELSE
	   'Escritura de archivo
	   LOCATE 10, 45: PRINT "Velpenet.dto"
	   OPEN "A:\CAP3\velpenet.dto" FOR OUTPUT AS #2
	   FOR KK = 1 TO M
		  LOCATE 14, 17
		  PRINT "Vel. de penetracin ("; KK; ") [pie/hr] = ";
		  INPUT "", VelPenetrac
		  LOCATE 14, 54
		  PRINT "          ": PRINT #2, VelPenetrac
	   NEXT
	END IF
    
	'Se abre PESBARRE.DTO para manejo de datos de peso sobre la barrena
	IF selec = 0 THEN
	   'Lectura de archivo
	   OPEN "A:\CAP3\pesbarre.dto" FOR INPUT AS #3
	   INPUT #3, Wdt
	   FOR KK = 1 TO M
		  INPUT #3, PesoSobreBarrena(KK)
	   NEXT
	ELSE
	   'Escritura de archivo
	   LOCATE 10, 45
	   PRINT "Pesbarre": OPEN "A:\CAP3\pesbarre.dto" FOR OUTPUT AS #3
	   LOCATE 14, 15
	   PRINT "Peso inicial sobre la bna. [1000 lb/pg] = "; : INPUT "", Wdt
	   LOCATE 14, 56
	   PRINT "        ": PRINT #3, Wdt
	   FOR KK = 1 TO M
		  LOCATE 14, 15
		  PRINT "Peso sobre la barrena ("; KK; ") [1000 lb/pg] = ";
		  INPUT "", PesoSobreBarrena
		  LOCATE 14, 15
		  PRINT "                                                    "
		  PRINT #3, PesoSobreBarrena
	   NEXT
	END IF
    
	'Se abre VELROTAC.DTO para manejo de datos de vel. de la mesa rotaria
	IF selec = 0 THEN
	   'Lectura de archivo
	   OPEN "A:\CAP3\velrotac.dto" FOR INPUT AS #4
	   FOR KK = 1 TO M
		  INPUT #4, VelRotac(KK)
	   NEXT
	ELSE
	   'Escritura de archivo
	   LOCATE 10, 45
	   PRINT "Velrotac"
	   OPEN "A:\CAP3\velrotac.dto" FOR OUTPUT AS #4
	   FOR KK = 1 TO M
		  LOCATE 14, 21
		  PRINT "Vel. de rotacin ("; KK; ") [rpm] = ";
		  INPUT "", VelRotac
		  LOCATE 14, 52
		  PRINT "           ": PRINT #4, VelRotac
	   NEXT
	END IF
    
	'Se abre DIENGAST.DTO para manejo de datos de desgaste de los dientes
	'de la barrena
	IF selec = 0 THEN
	   'Lectura de archivo
	   OPEN "A:\CAP3\diengast.dto" FOR INPUT AS #5
	   FOR KK = 1 TO M
		  INPUT #5, AltDientGastado(KK)
	   NEXT
	ELSE
	   'Escritura de archivo
	   LOCATE 10, 45
	   PRINT "Diengast"
	   OPEN "A:\CAP3\diengast.dto" FOR OUTPUT AS #5
	   FOR KK = 1 TO M
		  LOCATE 14, 17
		  PRINT "Altura de dientes gastados ("; KK; ") [frac] = ";
		  INPUT "", AltDientGastado
		  LOCATE 14, 17
		  PRINT "                                                  "
		  PRINT #5, AltDientGastado
	   NEXT
	END IF
    
	'Se abre REYNOLDS.DTO para manejo de datos del nm. de Reynolds
	IF selec = 0 THEN
	   'Lectura de archivo
	   OPEN "A:\CAP3\reynolds.dto" FOR INPUT AS #6
	   FOR KK = 1 TO M
		  INPUT #6, Reynolds(KK)
	   NEXT
	ELSE
	   'Escritura de archivo
	   LOCATE 10, 45
	   PRINT "Reynolds"
	   OPEN "A:\CAP3\reynolds.dto" FOR OUTPUT AS #6
	   FOR KK = 1 TO M
		  LOCATE 14, 21
		  PRINT "Nm. de Reynolds ("; KK; ") = "; : INPUT "", Reynolds
		  LOCATE 14, 46
		  PRINT "                    ": PRINT #6, Reynolds
	   NEXT
	END IF
    
	'Se abre DENSEQUI.DTO para manejo de datos de densidad equivalente
	IF selec = 0 THEN
	   'Lectura de archivo
	   OPEN "A:\CAP3\densequi.dto" FOR INPUT AS #7
	   FOR KK = 1 TO M
		  INPUT #7, DensEquivCirc(KK)
	   NEXT
	ELSE
	   'Escritura de archivo
	   LOCATE 10, 45
	   PRINT "Densequi": OPEN "A:\CAP3\densequi.dto" FOR OUTPUT AS #7
	   FOR KK = 1 TO M
		  LOCATE 14, 18
		  PRINT "Dens. equiv. de circ. ("; KK; ") [lb/gal] = ";
		  INPUT "", DensEquivCirc
		  LOCATE 14, 57
		  PRINT "          ": PRINT #7, DensEquivCirc
	   NEXT
	END IF
    
	'Se abre PRESPORO.DTO para manejo de datos de la presin de formacin
	IF selec = 0 THEN
	   'Lectura de archivo
	   OPEN "A:\CAP3\presform.dto" FOR INPUT AS #8
	   FOR KK = 1 TO M
		  INPUT #8, GradPresForm(KK)
	   NEXT
	ELSE
	   'Escritura de archivo
	   LOCATE 10, 45
	   PRINT "Presform"
	   OPEN "A:\CAP3\presform.dto" FOR OUTPUT AS #8
	   FOR KK = 1 TO M
		  LOCATE 14, 17
		  PRINT "Grad. de pres. de form. ("; KK; ") [lb/gal] = ";
		  INPUT "", GradPresForm
		  LOCATE 14, 58
		  PRINT "         ": PRINT #8, GradPresForm
	   NEXT
	END IF
    
	'Se cierran los archivos abiertos
	CLOSE #1, #2, #3, #4, #5, #6, #7, #8

END SUB

SUB marco (reni%, renf%, coli%, colf%, tipo%, ci%, cf%)
  
 'Subprograma que genera un marco
 
   'reni%, renf%-- Rengln inicial y final del recuadro
   'coli%, colf%-- Columna inicial y final del recuadro
   'tipo%       -- 1 para lnea sencilla, 2 para doble y 3 para semi-gruesa
   'ci%, cf%    -- Color inicial (recuadro) y final
 
   'Caractersticas del marco (tipo de lnea)
   SELECT CASE tipo%
		CASE 1
			'lnea sencilla
			esi$ = "": lhs$ = "": lv$ = "": esd$ = "": eii$ = ""
			lhi$ = "": eid$ = ""
		CASE 2
			'lnea doble
			esi$ = "": lhs$ = "": lv$ = "": esd$ = "": eii$ = ""
			lhi$ = "": eid$ = ""
		CASE 3
			'lnea semi-gruesa
			esi$ = "": lhs$ = "": lv$ = "": esd$ = "": eii$ = ""
			lhi$ = "": eid$ = ""
   END SELECT
 
   'Se despliega el recuadro
   COLOR ci%
	LOCATE reni%, coli%: PRINT esi$
	LOCATE reni%, colf%: PRINT esd$
	LOCATE reni%, coli% + 1: PRINT STRING$(colf% - coli% - 1, lhs$)
	FOR I% = reni% + 1 TO renf%
	    LOCATE I%, coli%: PRINT lv$
	    LOCATE I%, colf%: PRINT lv$
	NEXT I%
	LOCATE renf%, coli%: PRINT eii$
	LOCATE renf%, colf%: PRINT eid$
	LOCATE renf%, coli% + 1: PRINT STRING$(colf% - coli% - 1, lhi$)
   COLOR cf%

END SUB

SUB op1
 
  'Subrutina para control de la seleccin 1 del men principal
   
    CLS
   
    'Se invoca a la subrutina que genera un marco
	 CALL marco(2, 24, 2, 79, 3, 14, 0)
	 CALL marco(4, 7, 10, 70, 3, 2, 15)
   
    'Letreros
	 LOCATE 5, 13
	 PRINT "OBTENCION DE LOS COEFICIENTES DEL MODELO DE PERFORACION"
	 LOCATE 6, 27
	PRINT " USANDO LA SUBRUTINA SDV"
   
    'Se invoca a la subrutina que genera un marco
	 CALL marco(14, 16, 26, 52, 1, 12, 15)
   
    COLOR 31
	 LOCATE 15, 28: PRINT "Leyendo informacin ..."
    COLOR 0
   
    'Se invoca a la subrutina que maneja los datos de archivo (lectura)
	 CALL archi(0)
   
    COLOR 31
	 SOUND 290, 2
	 LOCATE 15, 28: PRINT "Efectuando clculos ..."
    COLOR 15
   
    'Generacin de la matriz de diseo
    FOR I = 1 TO M
	   A(I, 1) = 1
	  
	   'Efecto de compactacin
		A(I, 2) = 10000 - Profun(I)
		A(I, 3) = (Profun(I) ^ .69) * (GradPresForm(I) - 9)
	  
	   'Efecto de la presin diferencial
		A(I, 4) = Profun(I) * (GradPresForm(I) - DensEquivCirc(I))
	  
	   'Efecto del dimetro y el peso sobre la barrena
		A(I, 5) = LOG((PesoSobreBarrena(I) - Wdt) / (4 - Wdt))
	  
	   'Efecto de la velocidad de rotacin
		A(I, 6) = LOG(VelRotac(I) / 100)
	  
	   'Efecto del deterioro de los dientes de ls barrena
		A(I, 7) = -AltDientGastado(I)
	  
	   'Efecto del arrastre hidralico sobre la barrena
		A(I, 8) = LOG(Reynolds(I))
	  
	   'Valores de las ordenadas (vel. de penet., dD/dt)
		Y(I) = LOG(VelPenetrac(I))
    NEXT I
   
    'Se invoca a la subrutina SVD
	 CALL svd(NM, M, N, A(), SIGMA(), 1, U(), 1, V(), IERR, RV1())
   
    'Borrado de una seccin de pantalla
	FOR I = 1 TO 3
	    LOCATE 13 + I, 24: PRINT "                                "
	NEXT
   
    'Deteccin de error por SVD
    SOUND 300, 6
    IF IERR <> 0! THEN
	  SOUND 700, 6: SOUND 500, 3
	  COLOR 31
	    PRINT TAB(25); "ERROR ENCONTRADO POR SVD"
	  COLOR 0
	  'Se aborta el proceso
	  END
    END IF
   
    'Bsqueda del mximo y mnimo valor singular y deteccin de los valores
    'despreciables, adems, se inicializa el vector de coeficientes
	 SIGMAMAX = 0!
	 SIGMAMIN = SIGMA(1)
	 FOR J = 1 TO N
		IF SIGMA(J) > SIGMAMAX THEN SIGMAMAX = SIGMA(J)
		IF SIGMA(J) < SIGMAMIN THEN SIGMAMIN = SIGMA(J)
		C(J) = 0!
	 NEXT J
   
    'Nmero de condicin = mx/mn
	 cond = SIGMAMAX / SIGMAMIN
	 LOCATE 10, 26
	 PRINT "nm. de condicin ="; cond
   
    'Se proporciona un error de tolerancia y se calcula el error absoluto
    '(TAU)
	 RELERR = .001
	 TAU = RELERR * SIGMAMAX
   
    'Se calcula el vector de coeficientes C(J), de acuerdo a:   V c = U y
	 FOR J = 1 TO N
		IF SIGMA(J) <= TAU THEN GOTO c20
		S = 0!
		FOR I = 1 TO M
		    'Se evala y'= U^t y
			S = S + U(I, J) * Y(I)
		NEXT I
		'Se evala c'= ^-1 y'
		S = S / SIGMA(J)
		FOR I = 1 TO N
		   'Se evala c = V c'
		   C(I) = C(I) + S * V(I, J)
		NEXT I
c20:  NEXT J
   
    'Impresin de resultados
	 LOCATE 12, 23
	 PRINT "Coeficientes :"
	 FOR I = 1 TO N
		LOCATE 13 + I, 28
		PRINT "C ("; I; ") = "; C(I)
	 NEXT
   
    'Se evala el modelo en el punto TT mediante la tcnica de Horner
    '(aproximacin polinomial)
	 YY = 0!
	 FOR Jb = 1 TO N
		J = N + 1 - Jb
		YY = TT * YY + C(J)
	 NEXT
   
    'Se obtiene la raz cuadrada de la suma del cuadrado de los errores
    'residuales
	 RSQ = 0!
	 FOR I = 1 TO M
		RI = 0!
		FOR J = 1 TO N
		    RI = RI + C(J) * A(I, J)
		NEXT J
		RSQ = RSQ + (RI - Y(I)) ^ 2
	 NEXT I
	
    'Error residual incurrido
	 R = SQR(RSQ)
	 'PRINT TAB(15); "Error residual incurrido ="; R
	
    'Llamada al subrutina que funge como pausa
	 CALL pausa(24, 17, 30, 0)

END SUB

SUB op2
 
  'Subprograma que contiene los mandatos para la seleccin dos del
  'men principal
 
  CLS
  
   'Se invoca a la subrutina que ordena los datos en forma tabular
   CALL tabla(Profun(), VelPenetrac(), PesoSobreBarrena(), VelRotac(), AltDientGastado(), Reynolds(), DensEquivCirc(), GradPresForm())

END SUB

SUB op3
  
   'Subrutina para control de la seleccin 3 del men principal
    
    CLS
    LOCATE 3, , 0
   
    'Letrero
    PRINT TAB(6); "ESTA OPCION SOLO DEBE USARSE AL EJECUTAR EL PROGRAMA POR VEZ PRIMERA,"
    PRINT TAB(6); "DE LO CONTRARIO, SE CORRE EL RIESGO DE ELIMINAR LAS  BASES  DE  DATOS"
    PRINT TAB(6); "PREVIAMENTE CREADAS."
   
    'Llamada a la subrutina que genera un marco
	CALL marco(1, 24, 2, 79, 3, 14, 15)
   
    'Se ofrece la alternativa de abortar la instalacin
	COLOR 31
	  LOCATE 14, 21
	  PRINT "< ENTER > continuar, < ESC > terminar"
	COLOR 15
    
	DO
	  K$ = INKEY$
	LOOP WHILE K$ = ""
    
	'Transmutacin a cdigo ASCII
	 qy = ASC(RIGHT$(K$, 1))
    
	IF qy <> enter THEN EXIT SUB
    
	LOCATE 14, 21
	PRINT "                                        "
	LOCATE 10, 24
	PRINT "Creando el archivo : "
    
    'Llamada a la subrutina que genera un marco
	CALL marco(12, 16, 13, 67, 3, 6, 15)
    
    LOCATE 18, 20
    PRINT "Nota : archivos diseados para 30 datos"
    
    'Se invoca al archivo que maneja las bases de datos (para escritura)
	CALL archi(1)

END SUB

SUB opciones
  
   'Subrutina que facilita una seleccin de alternativas con el cursor
   'como apuntador
   
    CLS
   
    'Se invoca a la subrutina que genera un marco
	CALL marco(9, 15, 9, 72, 3, 2, 0)
	CALL marco(10, 14, 11, 70, 1, 2, 15)
   
    'Letreros
	LOCATE 11, 16: PRINT "Optimizacin de la perforacin mediante el clculo"
	LOCATE 12, 18: PRINT "de los coeficientes del modelo matemtico  que"
	LOCATE 13, 20: PRINT "la representa, empleando la subrutina  SVD"
   
    'Se invoca a la subrutina que activa una pausa
	CALL pausa(23, 19, 31, 0)
   
 '-------------- Despliegue grfico de opciones maestras -----------------

p10: CLS
    
	'Se invoca a la subrutina que genera un marco
	 CALL marco(8, 15, 11, 68, 3, 4, 15)
    
	'Letreros
	 LOCATE 8, 31:  PRINT " MENU PRINCIPAL "
	 LOCATE 10, 16: PRINT "_ Obtencin de los coeficientes."
	 LOCATE 11, 16: PRINT "_ Visualizar los datos reales de la perforacin."
	 LOCATE 12, 16: PRINT "_ Crear las bases de datos (instalacin)."
	 LOCATE 13, 16: PRINT "_ Trmino de la sesin."
	
	COLOR 31
	  LOCATE 20, 20: PRINT "    Seleccione la opcin de trabajo     "
	  LOCATE 21, 20: PRINT "        y   presione  < ENTER >         "
	COLOR 15
	
	BEEP
    
	'Se invoca a la subrutina que controla el apuntador
	 CALL apuntador(10, 13, 0, 16, FLAG1%)
    
	'Discriminacin de la opcin de trabajo
	 ON FLAG1% GOSUB p20, p30, p40, p50

p20: 'Opcin 1 (clculo de los coeficientes)
	 CALL op1
	 GOTO p10

p30: 'Opcin 2 (desplegado de datos de la perforacin)
	 CALL op2
	 GOTO p10

p40: 'Opcin 3 (subrutina que crea las bases de datos)
	 SOUND 500, 5
	 CALL op3
	 GOTO p10

p50: 'Opcin 4 (salida de sesin)
	 CLS
	
	 'Subrutina que despliega una torre
	  CALL torre
	
	 'Se invoca a la subrutina que genera un marco
	  CALL marco(22, 24, 52, 70, 2, 28, 0)
	
	 COLOR 31, 9
	   LOCATE 23, 54
	   PRINT " FIN DE SESION "
	 COLOR 1, 1, 1
	
	 'Se redefine la visibilidad del cursor
	  LOCATE , , 0
	
	 'Salida de sesin
	  END

END SUB

SUB pausa (ren%, col%, ci%, cf%)
   
  'Subprograma que funge como pausa
	
    'ren%, col% - Rengln y columna de desplegado
    'ci%, cf%   - Color inicial y final del mensaje
   
    'Letrero
    COLOR ci%
	 LOCATE ren%, col%
	 PRINT " < Para continuar presione cualquier tecla > "
    COLOR cf%
   
    'Ciclo que finaliza cuando se presiona cualquier tecla
    DO
    LOOP WHILE INKEY$ = ""

END SUB

SUB svd (NM, M, N, A(), W(), MATU, U(), MATV, V(), IERR, RV1())

    'Esta subrutina es una transferancia del programa SVD en lenguaje ALGOL.
    'Nm. Math. 14, 403-420 (1970), Golub y Reinsch, Handbook for  Automatic
    'Comp., Vol II-Linear Algebra, 134-151 (1971).

    'Esta rutina determina la descomposicin de los  valores  singulares  T,
    'A=USV de una matriz rectangular de M por N, se emplea una  bidiagonali-
    'lizacin comn y una variante del algoritmo QR.

    'ENTRADA:
    '  NM .. Debe ser el nm. de renglones de los  arreglos  bidimensionales
    '        que se declaran en la sentencia  de  llamado  a  la  subrutina.
    '        Ntese que NM debe ser cuando menos igual al mximo  valor  en-
    '        tre M y N.
    '  M ... Es el nmero de renglones de A (y de U), esto es, el nmero  de
    '        pares de datos base.
    '  N ... Es el nmero de columnas de A (y de U) y el orden de V. Adems,
    '        representa el grado del polinomio ms 1  para  la  aproximacin
    '        funcional, es decir, el nmero de coeficientes de  tal  funcin
    '        de aproximacin.
    '  A ... Representa la matriz rectangular de entrada, que  ser  descom-
    '        puesta.
    '  MATU. Debe asignarse el valor de 1, si en la descomposicin se  desea
    '        obtener la matriz U, y el de 0 en caso contrario.
    '  MATV. Debe asignarse el valor de 1, si en la descomposicin se  desea
    '        obtener la matriz V, y el de 0 en caso contrario.

    'SALIDA:
    '  A ... Permanece inalterada (a no ser que se reemplace por U o V).
    '  W ... Contiene los N valores singulares (no-negativos) de A  (elemen-
    '        tos de las diagonales de S). Los cuales estn desordenados.  Si
    '        se detecta un error automticamente se realiza la salida, y los
    '        valores singulares se corregirn por los ndices IERR+1, IERR+2,
    '        ..., N.
    '  U ... Contiene la matriz U (vectores columna ortogonales) de la  des-
    '        composicin si MATU=1, en caso contrario,  se  emplea  como  un
    '        arreglo temporal. Puede coincidir con la matriz A. Si se detecta
    '        un error, automticamente se realiza la salida, y  las columnas
    '        de U multiplicadas por los ndices de  correccin  ofrecen  los
    '        valores singulares efectivos.
    '  V ... Contiene la matriz V (ortogonal) de la descomposicin s MATV=1,
    '        en caso contrario, no se calcula. Puede coincidir con la matriz
    '        A si U no se requiere. Si se detecta un error,  automticamente
    '        se realiza la salida, y las columnas de V multiplicadas por los
    '        ndices de correccin ofrecen los valores singulares efectivos.
    '  IERR. Tendr el valor:
    '                        0 .... en una ejecucin normal.
    '                        K .... si el k-simo valor singular no se ha
    '                               obtenido despus de 30 iteraciones.
    '  RV1 . Es un arreglo para almacenaje temporal.

	 'Las interrogantes y comentarios, deben dirigirse a B. S. Garbow,
	 'Applied Mathematics Division, Argonne National Laboratory.

    'Rutina modificada para prescindir del MACHINE EPSILON.

    '  ------------------------------------------------------------------

	 IERR = 0
	 FOR I = 1 TO M
		FOR J = 1 TO N
		    U(I, J) = A(I, J)
		NEXT J
	 NEXT I

    'Reduccin a la forma bidiagonal
	 G = 0!
	 SCALE = 0!
	 ANORM = 0!
	 FOR I = 1 TO N
		L = I + 1
	    RV1(I) = SCALE * G
	    G = 0!
	    S = 0!
	    SCALE = 0!
	    IF I > M THEN GOTO Svd210
	    FOR K = I TO M
		   SCALE = SCALE + ABS(U(K, I))
	    NEXT K
	    IF SCALE = 0! GOTO Svd210
	    FOR K = I TO M
		   U(K, I) = U(K, I) / SCALE
		   S = S + U(K, I) ^ 2
	    NEXT K
	    F = U(I, I)
	    IF F >= 0! THEN G = -SQR(S) ELSE G = SQR(S)
	    H = F * G - S
	    U(I, I) = F - G
	    IF I = N GOTO Svd190
	    FOR J = L TO N
		   S = 0!
		   FOR K = I TO M
			  S = S + U(K, I) * U(K, J)
		   NEXT K
		   F = S / H
		   FOR K = I TO M
			  U(K, J) = U(K, J) + F * U(K, I)
		   NEXT K
	    NEXT J
Svd190:  FOR K = I TO M
		   U(K, I) = SCALE * U(K, I)
	    NEXT K
Svd210:  W(I) = SCALE * G
	    G = 0!
	    S = 0!
	    SCALE = 0!
	    IF I > M OR I = N GOTO Svd290
	    FOR K = L TO N
		   SCALE = SCALE + ABS(U(I, K))
	    NEXT K
	    IF SCALE = 0! GOTO Svd290
	    FOR K = L TO N
		   U(I, K) = U(I, K) / SCALE
		   S = S + U(I, K) ^ 2
	    NEXT K
	    F = U(I, L)
	    IF F >= 0! THEN G = -SQR(S) ELSE G = SQR(S)
	    H = F * G - S
	    U(I, L) = F - G
	    FOR K = L TO N
		   RV1(K) = U(I, K) / H
	    NEXT K
	    IF I = M GOTO Svd270
	    FOR J = L TO M
		   S = 0!
		   FOR K = L TO N
			  S = S + U(J, K) * U(I, K)
		   NEXT K
		   FOR K = L TO N
			  U(J, K) = U(J, K) + S * RV1(K)
		   NEXT K
	    NEXT J
Svd270:  FOR K = L TO N
		   U(I, K) = SCALE * U(I, K)
	    NEXT K
Svd290:  AUX = ABS(W(I)) + ABS(RV1(I))
	    IF AUX > ANORM THEN ANORM = AUX
	 NEXT I

    'Transformacin acumulativa hacia la derecha
	 IF MATV = 0 GOTO Svd410

    'Desde I = N hasta 1 , con decrementos de - 1
	 FOR II = 1 TO N
		I = N + 1 - II
		IF I = N GOTO Svd390
		IF G = 0! GOTO Svd360
		FOR J = L TO N
		    'Una doble divisin protege contra un posible error de
		    'underflow
			 V(J, I) = (U(I, J) / U(I, L)) / G
		NEXT J
		FOR J = L TO N
		    S = 0!
		    FOR K = L TO N
			   S = S + U(I, K) * V(K, J)
		    NEXT K
		    FOR K = L TO N
			   V(K, J) = V(K, J) + S * V(K, I)
		    NEXT K
		NEXT J
Svd360:   FOR J = L TO N
		    V(I, J) = 0!
		    V(J, I) = 0!
		NEXT J
Svd390:   V(I, I) = 1!
		G = RV1(I)
		L = I
	 NEXT II

    'Transformacin acumulativa hacia la izquierda
Svd410: IF MATU = 0 THEN GOTO Svd510

    'Desde I = mn.(M,N) hasta 1 , con decrementos de - 1
	 MN = N
	 IF M < N THEN MN = M
	 FOR II = 1 TO MN
		I = MN + 1 - II
		L = I + 1
		G = W(I)
		IF I = N GOTO Svd430
		FOR J = L TO N
		    U(I, J) = 0!
		NEXT J
Svd430:   IF G = 0! GOTO Svd475
		IF I = MN GOTO Svd460
		FOR J = L TO N
		    S = 0!
		    FOR K = L TO M
			   S = S + U(K, I) * U(K, J)
		    NEXT K
		    'Una doble divisin protege contra un posible error de
		    'underflow
			 F = (S / U(I, I)) / G
		    FOR K = I TO M
			   U(K, J) = U(K, J) + F * U(K, I)
		    NEXT K
		NEXT J
Svd460:   FOR J = I TO M
		    U(J, I) = U(J, I) / G
		NEXT J
		GOTO Svd490
Svd475:   FOR J = I TO M
		    U(J, I) = 0!
		NEXT J
Svd490:   U(I, I) = U(I, I) + 1!
	 NEXT II

    'Diagonalizacin de la forma bidiagonal desde K = N hasta 1 , con
    'decrementos de - 1
Svd510: FOR KK = 1 TO N
		  K1 = N - KK
		  K = K1 + 1
		  ITS = 0!
	   
		  'Prueba para la particin desde L = K hasta 1 , con
		  'decrementos de - 1
Svd520:         FOR LL = 1 TO K
				L1 = K - LL
				L = L1 + 1
				IF ABS(RV1(L)) + ANORM = ANORM GOTO Svd565
				'RV1(1) siempre es cero, por lo que no hay salida al
				'final del ciclo
				  IF ABS(W(L1)) + ANORM = ANORM THEN GOTO Svd540
			 NEXT LL
		
		    'Cancelacin de RV1(L) s L > 1
Svd540:         C = 0!
			 S = 1!
			 FOR I = L TO K
				F = S * RV1(I)
				RV1(I) = C * RV1(I)
				IF ABS(F) + ANORM = ANORM GOTO Svd565
				G = W(I)
				H = SQR(F ^ 2 + G ^ 2)
				W(I) = H
				C = G / H
				S = -F / H
				IF MATU = 0 THEN GOTO Svd560
				FOR J = 1 TO M
				    Y = U(J, L1)
				    Z = U(J, I)
				    U(J, L1) = Y * C + Z * S
				    U(J, I) = -Y * S + Z * C
				NEXT J
Svd560:         NEXT I
		
		    'Prueba de la convergencia
Svd565:         Z = W(K)
			 IF L = K GOTO Svd650
		
		    'Transferencia de 2 en 2 desde la parte inferior
			 IF ITS = 30 GOTO Svd1000
			 ITS = ITS + 1
			 X = W(L)
			 Y = W(K1)
			 G = RV1(K1)
			 H = RV1(K)
			 F = ((Y - Z) * (Y + Z) + (G - H) * (G + H)) / (2! * H * Y)
			 G = SQR(F ^ 2 + 1!)
			 IF F >= 0! THEN AUX = ABS(G) ELSE AUX = -ABS(G)
			 F = ((X - Z) * (X + Z) + H * (Y / (F + AUX) - H)) / X
		
		    'Siguiente transformacin QR
			 C = 1!
			 S = 1!
			 FOR I1 = L TO K1
				I = I1 + 1
				G = RV1(I)
				Y = W(I)
				H = S * G
				G = C * G
				Z = SQR(F ^ 2 + H ^ 2)
				RV1(I1) = Z
				C = F / Z
				S = H / Z
				F = X * C + G * S
				G = -X * S + G * C
				H = Y * S
				Y = Y * C
				IF MATV = 0 THEN GOTO Svd575
				FOR J = 1 TO N
				    X = V(J, I1)
				    Z = V(J, I)
				    V(J, I1) = X * C + Z * S
				    V(J, I) = -X * S + Z * C
				NEXT J
Svd575:             Z = SQR(F ^ 2 + H ^ 2)
				W(I1) = Z
		    
			   'La rotacin puede ser aleatoria si Z=0
				IF Z = 0! THEN GOTO Svd580
				C = F / Z
				S = H / Z
Svd580:             F = C * G + S * Y
				X = -S * G + C * Y
				IF MATU = 0 THEN GOTO Svd600
				FOR J = 1 TO M
				    Y = U(J, I1)
				    Z = U(J, I)
				    U(J, I1) = Y * C + Z * S
				    U(J, I) = -Y * S + Z * C
				NEXT J
Svd600:         NEXT I1
			 RV1(L) = 0!
			 RV1(K) = F
			 W(K) = X
			 GOTO Svd520
		
		    'Convergencia
Svd650:         IF Z >= 0! GOTO Svd700
		
		    'W(K) se hace no-negativo
			 W(K) = -Z
			 IF MATV = 0 GOTO Svd700
			 FOR J = 1 TO N
				V(J, K) = -V(J, K)
			 NEXT J
Svd700: NEXT KK
	   GOTO Svd1001
  
	 'Se presenta un error si no se converge hacia los valores
	 'singulares en 30 iteraciones
Svd1000: IERR = K
Svd1001: END SUB

SUB tabla (T1(), T2(), T3(), T4(), T5(), T6(), T7(), T8())
 
  'Subprograma que controla el despliegue tabular de los datos de
  'perforacin
   
    'Se invoca a la subrutina que despliega un recuadro
	 CALL marco(1, 24, 2, 79, 2, 12, 15)
    
    'Subproceso de encabezado
	 GOSUB encabezado
   
    'Se invoca a la subrutina que maneja los datos de archivo
	 CALL archi(0)
   
    'Desplegado de datos (1era. pantalla)
	FOR I = 1 TO 17
	    LOCATE 5 + I, 5
	    PRINT USING "##   #####   ##.#      #.##       ###     #.##   #.###   ##.#     ##.#"; I; T1(I); T2(I); T3(I); T4(I); T5(I); T6(I); T7(I); T8(I)
	NEXT
   
    LOCATE 23, 70: PRINT "Pg. 1/2"
   
    'Subrutina que funge como pausa temporal
	 CALL pausa(24, 30, 28, 0)
   
    CLS
   
    'Se invoca a la subrutina que despliega un recuadro
	 CALL marco(1, 24, 2, 79, 2, 12, 15)
   
    'Subproceso de encabezado
	 GOSUB encabezado
   
    'Desplegado de datos (2da. pantalla)
	FOR I = 1 TO 13
	    J = I + 17
	    LOCATE 5 + I, 5
	    PRINT USING "##   #####   ##.#      #.##       ###     #.##   #.###   ##.#     ##.#"; J; T1(J); T2(J); T3(J); T4(J); T5(J); T6(J); T7(J); T8(J)
	NEXT
   
    LOCATE 20, 6
    PRINT "Datos tomados del artculo `A Multiple Regression Approach to Optimal"
    LOCATE 21, 6
    PRINT "Drilling and Abnormal Pressure Detection', de Bourgoyne y Young, SPE,"
    LOCATE 22, 6
    PRINT "agosto, 1974."
    LOCATE 23, 70: PRINT "Pg. 2/2"
   
    'Llamada al subrutina que funge como pausa
	 CALL pausa(24, 30, 28, 0)
   
    EXIT SUB

encabezado:  'Subproceso que despliega el encabezado
   
    'Encabezado de la tabla
	 LOCATE 2, 4
	 PRINT " No.  Prof.   Vel.   Peso sobre    Vel.    Desg.  Nm.   Dens.    Grad. p."
	 LOCATE 3, 4
	 PRINT "Dato         penet.  la barrena   Rotac   diente  Reyn   equiv.   de form."
	 LOCATE 4, 4
	 PRINT "      [pie] [pie/hr] [1000lb/pg]  [rpm]   [frac]        [lb/gal]  [lb/gal]"
	 LOCATE 5, 4: PRINT STRING$(74, "")
   
    RETURN

END SUB

SUB torre
 
  'Subprograma que despliega en pantalla una torre de terminacin
  
    'Tubera
    COLOR 7, 9
	 LOCATE 2, 40: PRINT ""
    COLOR 15, 9
   
    'Estructrura de la torre
    PRINT TAB(35); ""
    PRINT TAB(35); "     "
    PRINT TAB(35); "   "
    PRINT TAB(35); "  "
    PRINT TAB(35); "    "
    PRINT TAB(35); "     "
    PRINT TAB(35); "   "
    PRINT TAB(35); "      "
   PRINT TAB(34); "   "
   PRINT TAB(34); "     "
   PRINT TAB(34); "         "
 PRINT TAB(32); " "
 PRINT TAB(32); "  Ĵ"
 PRINT TAB(32); "  Ĵ"

    'Estratos
    'COLOR 6, 1
    COLOR 15, 6
	 LOCATE 17, 1: PRINT STRING$(80, 176)
	 LOCATE 19, 1: PRINT STRING$(80, 177)

    'Tubera
    COLOR 7, 9
	 FOR I = 4 TO 13
		LOCATE I, 40: PRINT ""
	 NEXT
 
    COLOR 7, 0
	 FOR I = 15 TO 21
		LOCATE I, 40: PRINT ""
	 NEXT
   
	 'Tubera disparada
	 LOCATE 22, 40: PRINT ""
	 LOCATE 23, 40: PRINT ""
    COLOR 15, 9

    'Estructrura de la torre
    LOCATE 10, 40: PRINT ""
    LOCATE 14, 40: PRINT ""
 
END SUB

